# md5sum.tcl --
# 
# Part of: MHTCL
# Contents: example script for generating MD5 file signatures
# Date: Wed Jul  2, 2003
# 
# Abstract
# 
# 
# 
# Copyright (c) 2003, 2004 Marco Maggi
# 
# This is free  software you can redistribute it  and/or modify it under
# the terms of  the GNU General Public License as  published by the Free
# Software Foundation; either  version 2, or (at your  option) any later
# version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;  without   even  the   implied  warranty   of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

package require Tcl
package require mhash

set usage "usage:
   md5sum.tcl --create file file ... >sumfile
   md5sum.tcl --check  sumfile
"

#PAGE
# main --
#
#	Main procedure.
#
#  Arguments:
# 
#	argc -		The number of command line arguments.
#	argv -		The list of command line arguments.
#
#  Results:
#
#       Exits with code 0 or 1.
#

proc main { argc argv } {
    global	usage


    if { ! [mhash info hash exists md5] } {
	puts stderr "MHASH compiled with no support for md5"
	exit 1
    }

    if { $argc < 2 } {
	puts stderr $usage
	exit 1
    }

    switch -exact -- [lindex $argv 0] {
	--create	{
	    create_sums [lrange $argv 1 end]
	}
	--check		{
	    if { $argc != 2 } {
		puts stderr $usage
		exit
	    }
	    check_sums [lindex $argv end]
	}
	default		{
	    puts stderr $usage
	    exit 1
	}
    }

    exit 0
}
#PAGE
# create_sums --
#
#	Creates the digests for a set of files.
#
#  Arguments:
# 
#	argv -		The list of files.
#
#  Results:
#
#       Prints the digests on stdout.
#

proc create_sums { argv } {
    array set sums {}
    
    foreach filename $argv {
	set digest [make_sum $filename]
	set sums($filename) $digest
    }

    foreach filename [lsort [array names sums]] {
	puts stdout [format "%s  %s" $sums($filename) $filename]
    }
    return
}
#PAGE
# check_sums --
#
#	Reads a sum file and tests the digests in it.
#
#  Arguments:
# 
#	sumfile -	Pathname of the sum file.
#
#  Results:
#
#       Prints the result on stdout.
#

proc check_sums { sumfile } {
    set rex {^([a-fA-F0-9]+)  (.+)$}

    if { [catch {
	set id [open $sumfile { RDONLY }]
	set lst [read $id]
	close $id
    } res] } {
	puts stderr [format "error reading \"%s\": %s" $sumfile $res]
	exit 1
    }

    set lst [string trim $lst]
    foreach line [split $lst \n] {
	set line [string trim $line]

	if { ! [regexp -- $rex $line dummy key filename] } {
	    puts stderr "format error in sum file"
	    exit 1
	}
	set sums($filename) $key
    }

    foreach filename [lsort [array names sums]] {
	set key [make_sum $filename]

	if { [string equal $sums($filename) $key] } {
	    puts [format "correct authentication: \"%s\"" $filename]
	} else {
	    puts [format "authentication failed: \"%s\"" $filename]
	}
    }
    return
}
#PAGE
# make_sum --
#
#	Creates the digest for a file.
#
#  Arguments:
# 
#	filename -	The file pathname.
#
#  Results:
#
#       Returns the digest.
#

proc make_sum { filename } {
    set digest {}


    if { [catch {
	set channel [open $filename RDONLY]
	fconfigure $channel -translation binary
	set data [read $channel]
	close $channel
	
	set thread [mhash hash md5]
	fconfigure $thread -translation binary
	puts -nonewline $thread $data
	set digest [fconfigure $thread -digest]
	close $thread

	binary scan $digest H[expr {[string length $digest]*2}] digest
    } res] } {
	catch {close $channel}
	catch {close $thread}
	puts stderr [format "skipping %s: %s" $filename $res]
    }
    return $digest
}

main $argc $argv

### end of file
# Local Variables:
# mode: tcl
# page-delimiter: "^#PAGE"
# End:
