/* channel.c --
   
   Part of: MHTCL
   Contents: channel interface to the hash functions
   Date: Mon Jan 12, 2004
   
   Abstract
   
	This file defines the channel interface to hash functions.
   
   Copyright (c) 2004 Marco Maggi
   
   This is free software; you  can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software  Foundation; either version  2.1 of the License,  or (at
   your option) any later version.
   
   This library  is distributed in the  hope that it will  be useful, but
   WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
   MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
   Lesser General Public License for more details.
   
   You  should have  received a  copy of  the GNU  Lesser  General Public
   License along  with this library; if  not, write to  the Free Software
   Foundation, Inc.,  59 Temple Place,  Suite 330, Boston,  MA 02111-1307
   USA
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#include <string.h>
#include <errno.h>
#include <assert.h>
#include <stddef.h>
#include "mhtclInt.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define CHANNEL_NAME_LENGTH	(16 + TCL_INTEGER_SPACE)
#define CHANNEL_PATTERN		"mhtcl%u"


/** ------------------------------------------------------------
 ** Function prototypes.
 ** ----------------------------------------------------------*/

static Tcl_DriverCloseProc	MhtclChanClose;
static Tcl_DriverInputProc	MhtclChanInput;
static Tcl_DriverOutputProc	MhtclChanOutput;
static Tcl_DriverWatchProc	MhtclChanWatch;
static Tcl_DriverGetHandleProc	MhtclChanGetHandle;
static Tcl_DriverGetOptionProc	MhtclChanGetOption;

static Tcl_TimerProc		MhtclChanNotify;
static More_ChannelDriverGetOptionProc GetOptionDigest;


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct Channel {
  Tcl_Channel		channel;
  Mhtcl_Thread		thread;
  Tcl_TimerToken	timer;
  int			eventMask;
} Channel;


/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static Tcl_ChannelType Driver = {
  "mhtcl_hash",
  TCL_CHANNEL_VERSION_3,
  MhtclChanClose,
  MhtclChanInput,
  MhtclChanOutput,
  NULL, /* Seek */
  NULL, /* SetOption */
  MhtclChanGetOption,
  MhtclChanWatch,
  MhtclChanGetHandle,
  NULL, /* Close2 */
  NULL, /* Block */
  NULL, /* Flush */
  NULL, /* Handler */
  NULL /* WideSeek */
};

static Tcl_Mutex	counterMutex = NULL;
static int		channelCounter = 0;

static CONST More_ChannelDriverOption optionTable[] = {
  { "-digest", NULL, GetOptionDigest },
  { NULL, NULL, NULL }
};


More_Error
More_ErrorNewLogicStrInt (CONST char *errorInfo, int errorInt)
{
  More_Error	error;

  error = More_ErrorNew();
  More_ErrorLogicStr(error, errorInfo);
  More_ErrorSetInt(error, errorInt);
  return error;
}


/* Mhtcl_OpenHashChannel --

	Opens a new channel interface for a hash thread.

   Arguments:

	thread -	The hash thread descriptor.

   Results:

	Creates the channel and returns its token.

*/

Tcl_Channel
Mhtcl_OpenHashChannel (Mhtcl_Thread thread)
{
  char		channelName[CHANNEL_NAME_LENGTH];
  Channel *	data;


  debug("enter open hash channel: thread %p", thread);
  Tcl_MutexLock(&counterMutex);
  sprintf(channelName, CHANNEL_PATTERN, ++channelCounter);
  Tcl_MutexUnlock(&counterMutex);
  debug("channel name %s", channelName);

  data = (Channel *) ckalloc(sizeof(Channel));
  memset(data, '\0', sizeof(Channel));
  data->thread	= thread;
  data->channel = Tcl_CreateChannel(&Driver, channelName,
				    (ClientData) data, TCL_WRITABLE);
  debug("leaving open hash channel, success, channel %s, thread %p",
	Tcl_GetChannelName(data->channel), thread);
  return data->channel;
}

/* Mhtcl_MakeChannelHash --

	Builds  the hash  digest value  from  the current  state of  the
	channel.

   Arguments:

	channel -	The channel token.
	digestVar -	Pointer to a variable that will hold the
			digest object.

   Results:

        The hash  thread is cloned and terminated:  the resulting digest
        is  stored in  the selected  variable.  After  the call  to this
        function  other data  can  be  added to  the  channel state,  to
        produce other digests. Returns NULL or an error descriptor.

*/

More_Error
Mhtcl_MakeChannelHash (Tcl_Channel channel, Tcl_Obj **digestVar)
{
  Channel *	data;
  Mhtcl_Thread	clone;
  Tcl_Obj *	digest;
  More_Error	error;


  debug("entering digest finalisation for channel \"%p\"", channel);

  if (Tcl_GetChannelType(channel) != &Driver)
    {
      return More_ErrorNewLogicStrInt \
	("wrong channel type: not an mhtcl channel", EINVAL);
    }

  debug("flushing channel");

  if (Tcl_Flush(channel) != TCL_OK)
    {
      return More_ErrorNewLogicStrInt("error flushing data", Tcl_GetErrno());
    }

  data = (Channel *) Tcl_GetChannelInstanceData(channel);
  
  debug("duplicating thread %p", data->thread);

  error = Mhtcl_Copy(data->thread, &clone);
  if (error)
    {
      return error;
    }

  debug("duplicated thread: %p, now close it", clone);

  error = Mhtcl_Close(clone, &digest);
  if (error)
    {
      debug("error closing hash thread");
      Mhtcl_Abort(clone);
      return error;
    }

  *digestVar = digest;
  return NULL;
}

/* MhtclChanClose --

	Invoked  by  the  generic   layer  to  clean-up  driver  related
	informations when the channel is closed. 

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.

   Results:

	Terminates the  hash thread.  Must return zero  if the operation
        is  successful, or  a  non-zero  POSIX error  code  if an  error
        occurs: returns zero.

*/

int
MhtclChanClose (ClientData D, Tcl_Interp *interp)
{
  Channel *	data = (Channel *) D;

  debug("closing channel, aborting thread %p", data->thread);
  Mhtcl_Abort(data->thread);
  if (data->timer)
    {
      Tcl_DeleteTimerHandler(data->timer);
    }
  ckfree((char *) data);
  return 0;
}

/* MhtclChanInput --

	Invoked by  the generic layer to  read data from  the device and
	store it in an internal buffer.

   Arguments:

	D -		Pointer to the instance structure.
	outBuf -	Pointer to the output buffer.
	outLen -	Number of bytes to be read.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

        Must  return a  non-negative integer  indicating how  many bytes
        were read. In case of error  must return -1.  If an error occurs
        reading or writing  the variable: EINVAL is used  as POSIX error
        code.

*/

int
MhtclChanInput (ClientData D, char *outBuf, int outLen, int *errorCodePtr)
{
  return 0;
}

/* MhtclChanOutput --

	Invoked by the  generic layer to transfer data  from an internal
	buffer to the output device.

   Arguments:

	D -		Pointer to the instance structure.
	inBuf -		Pointer to the input buffer.
	inLen -		Number of bytes to be written.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

	Add the provided data to  the state of the channel.  Must return
        a   non-negative  integer   indicating  how   many   bytes  were
        written. In case of error must return -1.

*/

int
MhtclChanOutput (ClientData D, CONST char *inBuf, int inLen, int *errorCodePtr)
{
  Channel *	data = (Channel *) D;
  More_Block	block;


  debug("entering output for mhtcl thread \"%p\"", data->thread);
  debug("digesting %d bytes", inLen);

  block.len = inLen;
  block.ptr = (More_BytePtr) inBuf;
  Mhtcl_Digest(data->thread, block);

  debug("leaving output function, success");
  *errorCodePtr = 0;
  return inLen;
}

/* MhtclChanWatch --

	Invoked by the generic layer to initialise the event notification
	mechanism.

   Arguments:

	D -		Pointer to the instance data.
	mask -		An OR-ed combination of TCL_READABLE, TCL_WRITABLE
			and TCL_EXCEPTION. It indicates events the caller
			is interested in.

   Results:

        This channel is write-only and is always writable.

	  If the interest  in the writability is turned  off and a timer
        handler was  created: delete it. If the  interest in writability
        is turned on and no timer  was created: create one and store the
        timer token in the state structure.

	  The function associated to the handler will notify the generic
	layer about the  writability of the channel and  reset the timer
	token in the state structure.

*/

void
MhtclChanWatch (ClientData D, int mask)
{
  Channel *	data = (Channel *) D;


  debug("interests: writable %s", question(mask & TCL_WRITABLE));
  if (data->timer && (! (mask & TCL_WRITABLE)))
    {
      debug("deleting timer");
      Tcl_DeleteTimerHandler(data->timer);
      data->timer = (Tcl_TimerToken) NULL;
    }
  else if ((! data->timer) && (mask & TCL_WRITABLE))
    {
      debug("creating timer");
      data->timer = Tcl_CreateTimerHandler(1, MhtclChanNotify, data);
    }
}

/* MhtclChanNotify --

	Invoked  by  TCL when  the  timer  event  created by  the  watch
	function is processed.

   Arguments:

	D -		Pointer to the instance data.

   Results:

        Notifies the generic layer about the writability of the channel.

*/

void
MhtclChanNotify (ClientData D)
{
  Channel *	data = (Channel *) D;


  debug("notifying generic layer about writability");
  data->timer = NULL;
  Tcl_NotifyChannel(data->channel, TCL_WRITABLE);
}

/* MhtclChanGetHandle --

	Invoked by the generic layer to retrieve the device-specific
	handler for the channel.

   Arguments:

	D -		Pointer to the instance data.
	direction -	The channel direction: TCL_READABLE or
			TCL_WRITABLE.
	handlePtr -	Pointer to the variable that will hold
			the handler.

   Results:

        Returns TCL_ERROR.

*/

int
MhtclChanGetHandle (ClientData D, int direction, ClientData *handlePtr)
{
  return TCL_ERROR;
}

/* MhtclChanGetOption --

	Invoked by the generic layer  to retrieve the value of a channel
	specific option.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
MhtclChanGetOption (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  return More_ChannelDriverGetOption(optionTable, data->channel,
				     D, interp, optionName, optionValue);
}

/* GetOptionDigest --

	Duplicates  the thread  and finalises  it. Stores  the resulting
	hash in the output DString.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
GetOptionDigest (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;
  Tcl_Obj *	digObj;
  More_Error	error;


  error = Mhtcl_MakeChannelHash(data->channel, &digObj);
  if (error) { goto Error; }
  Tcl_IncrRefCount(digObj);
  Tcl_DStringAppend(optionValue, Tcl_GetString(digObj), -1);
  Tcl_DecrRefCount(digObj);
  return TCL_OK;

 Error:
  return More_ErrorResult(interp, error);
}

/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
