/* commands.c --
   
   Part of: MHTCL
   Contents: package commands
   Date: Wed Dec 25, 2002
   
   Abstract
   
	In this  file are  declared the callbacks  for the  TCL commands
	exposed in an interpreter in which the MHASH package is loaded.
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   This library is free software;  you can redistribute it and/or modify
   it  under the  terms  of the  GNU  Lesser General  Public License  as
   published by the Free Software  Foundation; either version 2.1 of the
   License, or (at your option) any later version.
   
   This library is  distributed in the hope that it  will be useful, but
   WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
   MERCHANTABILITY  or FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
   Lesser General Public License for more details.
   
   You  should have received  a copy  of the  GNU Lesser  General Public
   License along with  this library; if not, write  to the Free Software
   Foundation, Inc.,  59 Temple Place, Suite 330,  Boston, MA 02111-1307
   USA
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#ifdef MHTCL_STUB_TEST
#   include "mhtclTest.h"
#else
#   include "mhtclInt.h"
#endif

#include <limits.h>


/** ------------------------------------------------------------
 ** Include files.
 ** ----------------------------------------------------------*/

#include "declaration.c"

#ifndef MHTCL_STUB_TEST

int
Mhtcl_CreateCommands (Tcl_Interp *interp)
{
  More_Error	error;

  error = More_CreateCommands(interp, NULL, Mhtcl_CommandsTable);
  return ((error)? More_ErrorResult(interp, error) : TCL_OK);
}

#else

int
Mhtcltest_CreateCommands (Tcl_Interp *interp)
{
  More_Error	error;

  error = More_CreateCommands(interp, "test::mhashtest", Mhtcl_CommandsTable);
  return ((error)? More_ErrorResult(interp, error) : TCL_OK);
}

#endif


/* MhashHashFunction --

	Implementation function for the [mhash hash] command.

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Opens  a new  hash  channel. Returns  TCL_OK  or TCL_ERROR;  the
        return value of the TCL command is the channel's identifier.

*/

int
MhashHashFunction (More_CommandFrame frame)
{
  HashState *	state = (HashState *) frame->stateStructPtr;
  Mhtcl_Thread	thread;
  Tcl_Channel	channel;
  More_Error	error;


  error = Mhtcl_OpenHashThread(state->algorithm, &thread);
  if (error)
    {
      goto Error;
    }
  
  channel = Mhtcl_OpenHashChannel(thread);
  Tcl_RegisterChannel(frame->interp, channel);
  Tcl_SetResult(frame->interp, (char *) Tcl_GetChannelName(channel), TCL_STATIC);
  return TCL_OK;

 Error:
  return More_ErrorResult(frame->interp, error);
}

/* MhashHmacFunction --

	Implementation function for the [mhash hmac] command.

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Opens  a new  HMAC  channel. Returns  TCL_OK  or TCL_ERROR;  the
        return value of the TCL command is the channel's identifier.

*/

int
MhashHmacFunction (More_CommandFrame frame)
{
  HmacState *		state = (HmacState *) frame->stateStructPtr;
  Mhtcl_Thread		thread;
  Tcl_Channel		channel;
  More_Error		error;


  error = Mhtcl_OpenHMACThread(state->algorithm, state->password, &thread);
  if (error)
    {
      goto Error;
    }
  debug("interpreter %p, thread %p", frame->interp, thread);
  channel = Mhtcl_OpenHashChannel(thread);
  Tcl_RegisterChannel(frame->interp, channel);
  Tcl_SetResult(frame->interp, (char *) Tcl_GetChannelName(channel), TCL_STATIC);
  return TCL_OK;

 Error:
  return More_ErrorResult(frame->interp, error);
}

/* MhashKeygenFunction --

	Implementation     function    for     the     command    [mhash
	keygen]. Generates a key from a password seed.

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.


   Results:

        Returns TCL_OK or TCL_ERROR. The return value of the TCL command
        is the generated key, as a byte array.

*/

int
MhashKeygenFunction (More_CommandFrame frame)
{
  KeygenState *	state = (KeygenState *) frame->stateStructPtr;
  Tcl_Obj *	keyObj;
  More_Error	error;


  debug("enter key generation");
  error = Mhtcl_Keygen(state->algorithm, state->length, state->password,
		       state->hash1, state->hash2, state->count, state->salt,
		       &keyObj);
  if (!error)
    {
      Tcl_SetObjResult(frame->interp, keyObj);
      return TCL_OK;
    }
  else
    {
      return More_ErrorResult(frame->interp, error);
    }
}

/* MhashInfoHashExistsFunction --

	Implementation  function  for   the  command  [mhash  info  hash
	exists]. Tests if a hash algorithm is supported by the library.

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is true if the algorithm is supported, false otherwise.

*/

int
MhashInfoHashExistsFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj(Mhtcl_HashExists(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoKeygenExistsFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	exists]. Tests if a key generation algorithm is supported by the
	library.

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is true if the algorithm is supported, false otherwise.

*/
int
MhashInfoKeygenExistsFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj(Mhtcl_KeygenExists(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoHashListFunction --

	Implementation function for the command [mhash info hash list].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is the list of supported hash algorithm names.

*/

int
MhashInfoHashListFunction (More_CommandFrame frame)
{
  Tcl_SetObjResult(frame->interp, Mhtcl_ListHashAlgos());
  return TCL_OK;
}

/* MhashInfoKeygenListFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	list].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is the list of supported key generation algorithm names.

*/

int
MhashInfoKeygenListFunction (More_CommandFrame frame)
{
  Tcl_SetObjResult(frame->interp, Mhtcl_ListKeygenAlgos());
  return TCL_OK;
}

/* MhashInfoHashIshmacalgoFunction --

	Implementation  function  for   the  command  [mhash  info  hash
	isHMACalgo].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is true if the selected hash algorithm is usable for HMAC.

*/

int
MhashInfoHashIshmacalgoFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj(Mhtcl_IsHMACAlgo(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoHashBlocksizeFunction --

	Implementation  function  for   the  command  [mhash  info  hash
	blocksize].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is the number of bytes in the hash digest.

*/

int
MhashInfoHashBlocksizeFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewIntObj((int)Mhtcl_HashBlockSize(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoKeygenUsescountFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	usescount].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is true  if the selected  key generation algorithm makes  use of
        the "-count" option.

*/

int
MhashInfoKeygenUsescountFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj(Mhtcl_KeygenUsesCount(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoKeygenUsessaltFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	usessalt].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is true  if the selected  key generation algorithm makes  use of
        the "-salt" option.

*/

int
MhashInfoKeygenUsessaltFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj(Mhtcl_KeygenUsesSalt(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoKeygenSaltsizeFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	saltsize].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is  the number of  bytes required  in the  value of  the "-salt"
        option.

*/

int
MhashInfoKeygenSaltsizeFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewIntObj(Mhtcl_KeygenSaltSize(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoKeygenUseshashFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	useshash].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is 1 if  the selected key generation algorithm  makes use of the
        "-hash1" option, 2 if it makes use of the "-hash2" option, too.

*/

int
MhashInfoKeygenUseshashFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  debug("algo index %d (%p)", state->algorithm, &(state->algorithm));
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewIntObj(Mhtcl_KeygenUsesHash(state->algorithm)));
  return TCL_OK;
}


/* MhashInfoKeygenMaxkeysizeFunction --

	Implementation  function  for  the  command [mhash  info  keygen
	maxsize].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK or TCL_ERROR; the return value of the TCL command
        is  the maximum  size  of  the generated  key  for the  selected
        algorithm.

*/

int
MhashInfoKeygenMaxkeysizeFunction (More_CommandFrame frame)
{
  AlgorithmState *	state = (AlgorithmState *) frame->stateStructPtr;

  Tcl_SetObjResult(frame->interp,
		   Tcl_NewIntObj(Mhtcl_KeygenMaxKeySize(state->algorithm)));
  return TCL_OK;
}

/* MhashInfoVersionFunction --

	Implementation function for the command [mhash info version].

   Arguments:

	D -		Pointer to the invocation state structure.
	args -		Invocation arguments.

   Results:

        Returns TCL_OK; the return value  of the TCL command is a string
        representing the MHASH version.

*/

int
MhashInfoVersionFunction (More_CommandFrame frame)
{
  Tcl_SetObjResult(frame->interp, Tcl_NewIntObj(Mhtcl_MhashVersion()));
  return TCL_OK;
}


/* GetHashFromArg --

	Extracts a hash algorithm identifier from an argument.

*/

int
GetHashFromArg (More_ExtractorFrame frame)
{
  return Mhtcl_GetHashAlgFromObj(frame->commandFrame->interp,
				 frame->srcObj,
				 (int *) frame->fieldPtr);
}

/* GetBlockFromArg --

	Extracts a block from an argument.
*/

int
GetBlockFromArg (More_ExtractorFrame frame)
{
  return More_GetABlockFromArg(frame);
}

/* GetKeygenFromArg --

	Extracts a key generation algorithm identifier from an argument.

*/

int
GetKeygenFromArg (More_ExtractorFrame frame)
{
  debug("target var %p", (int *)frame->fieldPtr);
  return Mhtcl_GetKeygenAlgFromObj(frame->commandFrame->interp,
				   frame->srcObj,
				   (int *) frame->fieldPtr);
}

/* GetUnsignedFromArg

	Extracts  an unsigned  from  an  object. This  is  used for  the
	"length" and "count" fields of the KeygenState structure.
*/

int
GetUnsignedFromArg (More_ExtractorFrame frame)
{
  return More_GetUnsignedFromArg(frame);
}

/* end of file */
