/*
  This file is meant to be included (with #include) in a source
  file provided by you that implements the command functions. Your
  file must include the TCL header ("tcl.h") first and define the
  functions later.

    The reason of this inclusion is that here are declared the struct
  typedefs of the command invocation states. These types must be
  visible by the command functions. The stuff declared in this file
  uses the "static" keyword as much as possible.

    The command functions should be small: they should do the job by
  invoking other functions, the ones that the extension should export
  in its stub table.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "tclmore.h"



/** ------------------------------------------------------------
 ** Type declarations.
 ** ----------------------------------------------------------*/

/* HashState --

  This structure is used to hold the invocation state of the
  following commands:

                    [mhash hash   algorithm]
                    [mhash info hash blocksize   algorithm]
*/
typedef struct HashState {
  int                     algorithm;
} HashState;

static CONST HashState HashStateDefaults = {
  MHTCL_HASH_NONE
};

/* HmacState --

  This structure is used to hold the invocation state of the
  following commands:

                    [mhash hmac   algorithm password]
*/
typedef struct HmacState {
  int                     algorithm;
  More_Block              password;
} HmacState;

static CONST HmacState HmacStateDefaults = {
  MHTCL_HASH_NONE, MORE_BLOCK_NULL_VALUE
};

/* KeygenState --

  This structure is used to hold the invocation state of the
  following commands:

                    [mhash keygen   algorithm password length ?options?]
*/
typedef struct KeygenState {
  int                     algorithm;
  More_Block              password;
  int                     length;
  int                     hash1;
  int                     hash2;
  int                     count;
  More_Block              salt;
} KeygenState;

static CONST KeygenState KeygenStateDefaults = {
  MHTCL_KEYGEN_NONE, MORE_BLOCK_NULL_VALUE, 0, MHTCL_HASH_MD5, MHTCL_HASH_SHA1, 0, MORE_BLOCK_NULL_VALUE
};

/* DigestState --

  This structure is used to hold the invocation state of the
  following commands:


*/
typedef struct DigestState {
  Tcl_Channel             channel;
} DigestState;

static CONST DigestState DigestStateDefaults = {
  NULL
};

/* AlgorithmState --

  This structure is used to hold the invocation state of the
  following commands:

                    [mhash info hash exists   algorithm]
                    [mhash info hash ishmacalgo   algorithm]
                    [mhash info keygen exists   algorithm]
                    [mhash info keygen usescount   algorithm]
                    [mhash info keygen useshash   algorithm]
                    [mhash info keygen usessalt   algorithm]
                    [mhash info keygen saltsize   algorithm]
                    [mhash info keygen maxkeysize   algorithm]
*/
typedef struct AlgorithmState {
  int                     algorithm;
} AlgorithmState;

static CONST AlgorithmState AlgorithmStateDefaults = {
  MHTCL_HASH_NONE
};



/** ------------------------------------------------------------
 ** Implementation functions prototypes.
 ** ----------------------------------------------------------*/

/*
  You must provide the implementation of each of these functions in
  this file. When the function is invoked, the state invocation
  structure has been already filled with values from the command line
  arguments and options, with no errors.

    The function should look like the following:

  int
  MyCommandFunction (More_CommandFrame frame)
  {
     MyStateStruct *  state = (MyStateStruct *) frame->stateStructPtr;
     Tcl_Obj *        result;

     *
       Make use of the fields of "*state" to build a result by
       invoking functions from your library.
     *
     result = Mylib_DoSomething(state->value, state->another_value);

     Tcl_SetObjResult(frame->interp, result);
     return TCL_OK;
  }

    Being the implementation of a TCL command, this function is
  declared "static": it is useless to share this function among
  different modules. Remember that the role of a "More_CmdFunc"
  is to be an interface between the state invocation structure and
  the functions of your library that do the "real work".
*/

static More_CmdFunc         MhashHashFunction;
static More_CmdFunc         MhashHmacFunction;
static More_CmdFunc         MhashKeygenFunction;
static More_CmdFunc           MhashInfoHashExistsFunction;
static More_CmdFunc           MhashInfoHashListFunction;
static More_CmdFunc           MhashInfoHashIshmacalgoFunction;
static More_CmdFunc           MhashInfoHashBlocksizeFunction;
static More_CmdFunc           MhashInfoKeygenExistsFunction;
static More_CmdFunc           MhashInfoKeygenListFunction;
static More_CmdFunc           MhashInfoKeygenUsescountFunction;
static More_CmdFunc           MhashInfoKeygenUseshashFunction;
static More_CmdFunc           MhashInfoKeygenUsessaltFunction;
static More_CmdFunc           MhashInfoKeygenSaltsizeFunction;
static More_CmdFunc           MhashInfoKeygenMaxkeysizeFunction;
static More_CmdFunc          MhashInfoVersionFunction;


/** ------------------------------------------------------------
 ** Extractor functions prototypes.
 ** ----------------------------------------------------------*/

/*
  You must provide the implementation of each of the static extractors
  in this file. Functions declared EXTERN can be implemented in other
  modules of your extension or in other libraries that are dependencies
  of this one (for example the TCLMORE library implements some common
  extractors).

    The functions must be available as pointers: you cannot directly use
  functions exported in a stub table; this is because the pointer must
  be included in a statically allocated data structure. Remember that
  stub function names are available as macros, each macro references
  an element in a vector of function pointers in another library: the
  value of the function pointer is available only at execution time.

    The only way to use an extractor from a stub table is to declare
  another extractor (with a similar name) that wraps the real function.
  Example:

     More_Extractor MyExtractor;
     ...
     int
     MyExtractor (More_ExtractorFrame frame)
     {
       return RealExtractor(frame);
     }

  "RealExtractor" can be the macro of a stub function.

    If a prototype is omitted: it is your responsibility to declare it
  before the line that includes this file in your source module. Obviously
  the declaration can be in an external header file; this is particularly
  useful if your extension is using a wrapper extractor for a real extractor
  from another extension, and the extractor is used in more than one
  set of commands.

    A common extractor has the following structure, lets assume that
  "Type" is the type of value we want:

     EXTERN int Mylib_GetTypeFromObj _ANSI_ARGS_((Tcl_Interp * interp,
                                                  Tcl_Obj *    srcObj,
                                                  Type *       valueVar));
     ...
     int
     TypeExtractor (More_ExtractorFrame frame)
     {
        return Mylib_GetTypeFromObj(frame->commandFrame->interp,
                                    frame->srcObj,
                                    (Type *) frame->fieldPtr);       
     }

  or:

     int
     TypeExtractor (More_ExtractorFrame frame)
     {
        int   e;

        e = Mylib_GetTypeFromObj(frame->commandFrame->interp,
                                 frame->srcObj,
                                 (Type *) frame->fieldPtr);       
        return ((TCL_OK == e)? e : More_LogicError(interp));
     }

  the implementation of a "GetTypeFromObj" function to do the real
  work is recommended because such functions can be used by other
  modules, and even exported in the stub table.

    Remember that the field pointer can reference a whole structure
  embedded in the state invocation one.

    At present an extractor can validate arguments only with complete
  independence from the current process state, that is it can test that
  a value is in the correct space and range. Coherence tests between
  argument values and process state must be performed by the command
  implementation functions. This is not exactly true, but it is the
  recommended way to do things.
*/

static More_Extractor      GetHashFromArg;
static More_Extractor      GetBlockFromArg;
static More_Extractor      GetKeygenFromArg;
static More_Extractor      GetUnsignedFromArg;


/** ------------------------------------------------------------
 ** Command arguments tables.
 ** ----------------------------------------------------------*/

#define mhash_hash_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashHashArguments[1] = {
  { GetHashFromArg,      offsetof(struct HashState, algorithm) }
};

#define mhash_hmac_ARGUMENT_NUMBER	2
static CONST More_CommandArgument MhashHmacArguments[2] = {
  { GetHashFromArg,      offsetof(struct HmacState, algorithm) },
  { GetBlockFromArg,     offsetof(struct HmacState, password) }
};

#define mhash_keygen_ARGUMENT_NUMBER	3
static CONST More_CommandArgument MhashKeygenArguments[3] = {
  { GetKeygenFromArg,    offsetof(struct KeygenState, algorithm) },
  { GetBlockFromArg,     offsetof(struct KeygenState, password) },
  { GetUnsignedFromArg,  offsetof(struct KeygenState, length) }
};

#define mhash_info_hash_exists_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoHashExistsArguments[1] = {
  { GetHashFromArg,      offsetof(struct AlgorithmState, algorithm) }
};

/* No arguments for [mhash info hash list]. */
#define mhash_info_hash_list_ARGUMENT_NUMBER	0

#define mhash_info_hash_ishmacalgo_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoHashIshmacalgoArguments[1] = {
  { GetHashFromArg,      offsetof(struct AlgorithmState, algorithm) }
};

#define mhash_info_hash_blocksize_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoHashBlocksizeArguments[1] = {
  { GetHashFromArg,      offsetof(struct HashState, algorithm) }
};

#define mhash_info_keygen_exists_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoKeygenExistsArguments[1] = {
  { GetKeygenFromArg,    offsetof(struct AlgorithmState, algorithm) }
};

/* No arguments for [mhash info keygen list]. */
#define mhash_info_keygen_list_ARGUMENT_NUMBER	0

#define mhash_info_keygen_usescount_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoKeygenUsescountArguments[1] = {
  { GetKeygenFromArg,    offsetof(struct AlgorithmState, algorithm) }
};

#define mhash_info_keygen_useshash_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoKeygenUseshashArguments[1] = {
  { GetKeygenFromArg,    offsetof(struct AlgorithmState, algorithm) }
};

#define mhash_info_keygen_usessalt_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoKeygenUsessaltArguments[1] = {
  { GetKeygenFromArg,    offsetof(struct AlgorithmState, algorithm) }
};

#define mhash_info_keygen_saltsize_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoKeygenSaltsizeArguments[1] = {
  { GetKeygenFromArg,    offsetof(struct AlgorithmState, algorithm) }
};

#define mhash_info_keygen_maxkeysize_ARGUMENT_NUMBER	1
static CONST More_CommandArgument MhashInfoKeygenMaxkeysizeArguments[1] = {
  { GetKeygenFromArg,    offsetof(struct AlgorithmState, algorithm) }
};

/* No arguments for [mhash info version]. */
#define mhash_info_version_ARGUMENT_NUMBER	0



/** ------------------------------------------------------------
 ** Command options tables.
 ** ----------------------------------------------------------*/

/* No options for [mhash hash]. */

/* No options for [mhash hmac]. */

static CONST More_CommandOption MhashKeygenOptions[5] = {
  { "-hash1", 1, GetHashFromArg, offsetof(struct KeygenState, hash1) },
  { "-hash2", 1, GetHashFromArg, offsetof(struct KeygenState, hash2) },
  { "-salt", 1, GetBlockFromArg, offsetof(struct KeygenState, salt) },
  { "-count", 1, GetUnsignedFromArg, offsetof(struct KeygenState, count) },
  { NULL, 0, NULL, 0 }
};

/* No options for [mhash info hash exists]. */

/* No options for [mhash info hash list]. */

/* No options for [mhash info hash ishmacalgo]. */

/* No options for [mhash info hash blocksize]. */

/* No options for [mhash info keygen exists]. */

/* No options for [mhash info keygen list]. */

/* No options for [mhash info keygen usescount]. */

/* No options for [mhash info keygen useshash]. */

/* No options for [mhash info keygen usessalt]. */

/* No options for [mhash info keygen saltsize]. */

/* No options for [mhash info keygen maxkeysize]. */

/* No options for [mhash info version]. */


/** ------------------------------------------------------------
 ** Leaf commands array.
 ** ----------------------------------------------------------*/

/*
  There is a leaf structure for each "More_CmdFunc". Each structure
  has references to the argument and option tables. It is not your
  business to mess with this structure.
*/

static More_CommandTreeLeaf commandTreeLeaves[] = {
  /* (0) synopsis: mhash hash algorithm */
  { 2, MhashHashFunction, sizeof(HashState), (CONST ClientData)&HashStateDefaults,
    mhash_hash_ARGUMENT_NUMBER, MhashHashArguments, NULL,
    "algorithm" },

  /* (1) synopsis: mhash hmac algorithm password */
  { 2, MhashHmacFunction, sizeof(HmacState), (CONST ClientData)&HmacStateDefaults,
    mhash_hmac_ARGUMENT_NUMBER, MhashHmacArguments, NULL,
    "algorithm password" },

  /* (2) synopsis: mhash keygen algorithm password length ?options? */
  { 2, MhashKeygenFunction, sizeof(KeygenState), (CONST ClientData)&KeygenStateDefaults,
    mhash_keygen_ARGUMENT_NUMBER, MhashKeygenArguments, MhashKeygenOptions,
    "algorithm password length ?options?" },

  /* (3) synopsis: mhash info hash exists algorithm */
  { 4, MhashInfoHashExistsFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_hash_exists_ARGUMENT_NUMBER, MhashInfoHashExistsArguments, NULL,
    "algorithm" },

  /* (4) synopsis: mhash info hash list  */
  { 4, MhashInfoHashListFunction, 0, (CONST ClientData)NULL,
    mhash_info_hash_list_ARGUMENT_NUMBER, NULL, NULL,
    NULL },

  /* (5) synopsis: mhash info hash ishmacalgo algorithm */
  { 4, MhashInfoHashIshmacalgoFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_hash_ishmacalgo_ARGUMENT_NUMBER, MhashInfoHashIshmacalgoArguments, NULL,
    "algorithm" },

  /* (6) synopsis: mhash info hash blocksize algorithm */
  { 4, MhashInfoHashBlocksizeFunction, sizeof(HashState), (CONST ClientData)&HashStateDefaults,
    mhash_info_hash_blocksize_ARGUMENT_NUMBER, MhashInfoHashBlocksizeArguments, NULL,
    "algorithm" },

  /* (7) synopsis: mhash info keygen exists algorithm */
  { 4, MhashInfoKeygenExistsFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_keygen_exists_ARGUMENT_NUMBER, MhashInfoKeygenExistsArguments, NULL,
    "algorithm" },

  /* (8) synopsis: mhash info keygen list  */
  { 4, MhashInfoKeygenListFunction, 0, (CONST ClientData)NULL,
    mhash_info_keygen_list_ARGUMENT_NUMBER, NULL, NULL,
    NULL },

  /* (9) synopsis: mhash info keygen usescount algorithm */
  { 4, MhashInfoKeygenUsescountFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_keygen_usescount_ARGUMENT_NUMBER, MhashInfoKeygenUsescountArguments, NULL,
    "algorithm" },

  /* (10) synopsis: mhash info keygen useshash algorithm */
  { 4, MhashInfoKeygenUseshashFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_keygen_useshash_ARGUMENT_NUMBER, MhashInfoKeygenUseshashArguments, NULL,
    "algorithm" },

  /* (11) synopsis: mhash info keygen usessalt algorithm */
  { 4, MhashInfoKeygenUsessaltFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_keygen_usessalt_ARGUMENT_NUMBER, MhashInfoKeygenUsessaltArguments, NULL,
    "algorithm" },

  /* (12) synopsis: mhash info keygen saltsize algorithm */
  { 4, MhashInfoKeygenSaltsizeFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_keygen_saltsize_ARGUMENT_NUMBER, MhashInfoKeygenSaltsizeArguments, NULL,
    "algorithm" },

  /* (13) synopsis: mhash info keygen maxkeysize algorithm */
  { 4, MhashInfoKeygenMaxkeysizeFunction, sizeof(AlgorithmState), (CONST ClientData)&AlgorithmStateDefaults,
    mhash_info_keygen_maxkeysize_ARGUMENT_NUMBER, MhashInfoKeygenMaxkeysizeArguments, NULL,
    "algorithm" },

  /* (14) synopsis: mhash info version  */
  { 3, MhashInfoVersionFunction, 0, (CONST ClientData)NULL,
    mhash_info_version_ARGUMENT_NUMBER, NULL, NULL,
    NULL }
};


/** ------------------------------------------------------------
 ** Node commands array.
 ** ----------------------------------------------------------*/

/*
  These are node structures that declare subcommands with subcommands.
  It is not your business to mess with this structure.
*/

static More_CommandTreeNode MhashInfoKeygenCommandNode[] = {
  { "exists", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[7] },
  { "list", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[8] },
  { "usescount", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[9] },
  { "useshash", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[10] },
  { "usessalt", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[11] },
  { "saltsize", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[12] },
  { "maxkeysize", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[13] },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};

static More_CommandTreeNode MhashInfoHashCommandNode[] = {
  { "exists", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[3] },
  { "list", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[4] },
  { "ishmacalgo", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[5] },
  { "blocksize", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[6] },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};

static More_CommandTreeNode MhashInfoCommandNode[] = {
  { "hash", TCLMORE_SAFE_COMMAND,  MhashInfoHashCommandNode, NULL },
  { "keygen", TCLMORE_SAFE_COMMAND,  MhashInfoKeygenCommandNode, NULL },
  { "version", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[14] },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};

static More_CommandTreeNode MhashCommandNode[] = {
  { "hash", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[0] },
  { "hmac", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[1] },
  { "keygen", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[2] },
  { "info", TCLMORE_SAFE_COMMAND,  MhashInfoCommandNode, NULL },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};



/** ------------------------------------------------------------
 ** Root commands table.
 ** ----------------------------------------------------------*/

/*
  This array of structures declares the root commands. There is one of
  them for each <maincommand> and <command> markups with no ancestors
  in the original XML document. It is not your business to mess with
  the content of this array.

    You may create commands declared in this array with the following
  chunk of code:

    error = More_CreateCommands(interp, "namesp", Mhtcl_CommandsTable);
    if (error)
      {
	return More_ErrorResult(interp, error);
      }

  where "namesp" is the name of the namespace in which the command
  must be created. If the namespace is the root one: you can use NULL.
*/

More_CommandTreeNode Mhtcl_CommandsTable[] = {
  { "mhash", TCLMORE_SAFE_COMMAND, MhashCommandNode, NULL },
  { NULL, 0, NULL, NULL }
};


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/

