/* interface.c --
   
   Part of: MHTCL
   Contents: digest threads handling
   Date: Wed Dec 25, 2002
   
   Abstract
   
	This file  defines the  interface to MHASH.  It's the  only file
	exposed to the "mhash.h" header file.
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   This library is free software;  you can redistribute it and/or modify
   it  under the  terms  of the  GNU  Lesser General  Public License  as
   published by the Free Software  Foundation; either version 2.1 of the
   License, or (at your option) any later version.
   
   This library is  distributed in the hope that it  will be useful, but
   WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
   MERCHANTABILITY  or FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
   Lesser General Public License for more details.
   
   You  should have received  a copy  of the  GNU Lesser  General Public
   License along with  this library; if not, write  to the Free Software
   Foundation, Inc.,  59 Temple Place, Suite 330,  Boston, MA 02111-1307
   USA
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
# define TCLMORE_DEBUG_MESSAGES
# define TCLMORE_DEBUGGING	1
#endif

#define MHTCL_THREAD_TYPEDEF	1
struct Thread;
typedef struct Thread *		Mhtcl_Thread;

#include "mhtclInt.h"
#include "mhash.h"
#include <errno.h>
#include <stdlib.h>
#include <assert.h>


/** ------------------------------------------------------------
 ** Preprocessor macros.
 ** ----------------------------------------------------------*/

#define HASH_THREAD		0
#define HMAC_THREAD		1

#define NotHashAlgo		((algorithm < 0) || \
				 (MHTCL_HASH_INVALID <= algorithm))

#define NotKeygenAlgo		((algorithm < 0) || \
				 (MHTCL_KEYGEN_INVALID <= algorithm))


/** ------------------------------------------------------------
 ** Type declarations.
 ** ----------------------------------------------------------*/

typedef struct Thread {
  int		running;
  int		type;
  int		algorithm;
  MHASH		data;
  int		result_length;
} Thread;

typedef struct HashSelector {
  CONST char *	name;
  hashid	id;
} HashSelector;

typedef struct KeygenSelector {
  CONST char *	name;
  keygenid	id;
} KeygenSelector;

/*
  The following is for debug purposes only.
*/

typedef struct mhash_instance_t {
  int hmac_key_size;
  int hmac_block;
  unsigned char *hmac_key;

  unsigned char *state;
  int	state_size;
  hashid algorithm_given;
  
  void * hash_func;
  void * final_func;
  void * deinit_func;
	                        
} mhash_instance_t;


/** ------------------------------------------------------------
 ** Global variables.
 ** ----------------------------------------------------------*/

/* do not modify the order when adding new algorithms */
static CONST HashSelector HashAlgos[] = {
  { "adler32", MHASH_ADLER32 },
  { "crc32", MHASH_CRC32 },
  { "crc32b", MHASH_CRC32B },
  { "gost", MHASH_GOST },
  { "haval128", MHASH_HAVAL128 },
  { "haval160", MHASH_HAVAL160 },
  { "haval192", MHASH_HAVAL192 },
  { "haval224", MHASH_HAVAL224 },
  { "haval256", MHASH_HAVAL256 },
  { "md4", MHASH_MD4 },
  { "md5", MHASH_MD5 },
  { "ripemd160", MHASH_RIPEMD160 },
  { "sha1", MHASH_SHA1 },
  { "sha256", MHASH_SHA256 },
  { "tiger", MHASH_TIGER },
  { "tiger128", MHASH_TIGER128 },
  { "tiger160", MHASH_TIGER160 },
  /* added: Sat May  1, 2004 */
  { "ripemd128", MHASH_RIPEMD128 },
  { "ripemd256", MHASH_RIPEMD256 },
  { "ripemd320", MHASH_RIPEMD320 },
  { "sha224", MHASH_SHA224 },
  { "sha384", MHASH_SHA384 },
  { "sha512", MHASH_SHA512 },
  { "whirlpool", MHSH_WHIRLPOOL },
/*   { "whirlpool", MHASH_WHIRLPOOL }, */

  { NULL, MHASH_TIGER }
};

/*
  do not modify the order when adding new algorithms
*/
static CONST KeygenSelector KeygenAlgos[] = {
  { "asis", KEYGEN_ASIS },
  { "hex", KEYGEN_HEX },
  { "mcrypt", KEYGEN_MCRYPT },
  { "pkdes", KEYGEN_PKDES },
  { "s2k_isalted", KEYGEN_S2K_ISALTED },
  { "s2k_salted", KEYGEN_S2K_SALTED },
  { "s2k_simple", KEYGEN_S2K_SIMPLE },
  { NULL, KEYGEN_ASIS }
};

#ifdef TCLMORE_DEBUGGING

void
PrintMhash (MHASH data)
{
  mhash_instance_t *	p = (mhash_instance_t *) data;

  debug("mhash instance:");
  debug("algorithm: %d", p->algorithm_given);
  debug("state size: %d", p->state_size);
  debug("key size: %d", p->hmac_key_size);
  debug("key: %p", p->hmac_key);
  debug("funcs: %p %p %p", p->hash_func, p->final_func, p->deinit_func);
}

#else
#  define PrintMhash(data)
#endif


/* Mhtcl_OpenHashThread --

	Creates and initialises a new hash thread.

   Arguments:

	algorithm -	The algorithm selector.
	threadVar -	Pointer to a variable that will hold
			the thread descriptor.

   Results:

        Allocates and initialises the  new thread structure, then stores
        a pointer  to it  in the supplied  variable. Returns NULL  or an
        error descriptor.

	  Inspection of "mhash_init()"  from "lib/mhash.c" in the source
	code of MHASH version 0.8.18  reveals that the function can fail
	only  if the algorithm  selector is  wrong or  memory allocation
	fails. We check the selector before the call, so the only reason
	for error is memory allocation.

*/

More_Error
Mhtcl_OpenHashThread (int algorithm, Mhtcl_Thread *threadVar)
{
  Mhtcl_Thread	thread;
  MHASH		data;
  int		errorCode;
  CONST char *	errorString;
  More_Error	error;


  debug("opening hash thread, algorithm %d", algorithm);
  if (NotHashAlgo)
    {
      errorCode   = EINVAL;
      errorString = "invalid algorithm selector";
      goto Error;
    }

  data = mhash_init(HashAlgos[algorithm].id);
  if (MHASH_FAILED == data)
    {
      error = More_ErrorNoMemory();
      More_ErrorSetInt(error, ENOMEM);
      return error;
    }

  thread = (Mhtcl_Thread) ckalloc(sizeof(Thread));
  thread->running	= 0;
  thread->type		= HASH_THREAD;
  thread->algorithm	= algorithm;
  thread->data		= data;
  thread->result_length	= Mhtcl_HashBlockSize(algorithm);

  *threadVar = thread;
  debug("built thread %p, result size %d", *threadVar, thread->result_length);
  PrintMhash(thread->data);
  debug("exiting open hash thread, return success");
  return NULL;

 Error:
  error = More_ErrorNew();
  More_ErrorLogicStr(error, errorString);
  More_ErrorSetInt(error, errorCode);
  return error;
}

/* Mhtcl_OpenHMACThread --

	Creates and initialises a new HMAC thread.

   Arguments:

	algorithm -	The algorithm selector.
	password -	The password block.
	threadVar -	Pointer to a variable that will hold
			the thread descriptor.

   Results:

        Allocates and initialises the  new thread structure, then stores
        a pointer  to it  in the supplied  variable. Returns NULL  or an
        error descriptor.

	  Inspection  of "mhash_hmac_init()"  from "lib/mhash.c"  in the
	source code  of MHASH version  0.8.18 reveals that  the function
	can  fail only  if the  algorithm  selector is  wrong or  memory
	allocation fails. We check the  selector before the call, so the
	only reason for error is memory allocation.

*/

More_Error
Mhtcl_OpenHMACThread(algorithm, password, threadVar)
     int		algorithm;
     More_Block		password;
     Mhtcl_Thread *	threadVar;
{
  int		pblock;
  Mhtcl_Thread	thread;
  MHASH		data;
  More_Error	error;
  int		errorCode;
  CONST char *	errorString;


  if (NotHashAlgo)
    {
      errorCode   = EINVAL;
      errorString = "invalid algorithm selector";
      goto Error;
    }

  if (! Mhtcl_IsHMACAlgo(algorithm))
    {
      errorCode   = EINVAL;
      errorString = "invalid hash algorithm for HMAC";
      goto Error;
    }

  if ((! password.len) || (! password.ptr))
    {
      errorCode   = EINVAL;
      errorString = "null password";
      goto Error;
    }

  pblock = mhash_get_hash_pblock(HashAlgos[algorithm].id);
  data = mhash_hmac_init(HashAlgos[algorithm].id,
			 (void *)password.ptr, password.len, pblock);
  if (MHASH_FAILED == data)
    {
      error = More_ErrorNoMemory();
      More_ErrorSetInt(error, ENOMEM);
      return error;
    }

  thread = (Mhtcl_Thread) ckalloc(sizeof(Thread));
  thread->running	= 0;
  thread->type		= HMAC_THREAD;
  thread->algorithm	= algorithm;
  thread->data		= data;
  thread->result_length	= Mhtcl_HashBlockSize(algorithm);

  *threadVar = thread;
  return NULL;

 Error:
  error = More_ErrorNew();
  More_ErrorLogicStr(error, errorString);
  More_ErrorSetInt(error, errorCode);
  return error;
}

/* Mhtcl_Digest --

	Digests a block of data adding it to the context of a thread.

   Arguments:

	thread -	The thread descriptor.
	block -		The block of data to process.

   Results:

        Returns nothing.

*/

void
Mhtcl_Digest (Mhtcl_Thread thread, More_Block block)
{
  debug("processing data, thread %p", thread);
  PrintMhash(thread->data);
  mhash(thread->data, block.ptr, (size_t)block.len);
  PrintMhash(thread->data);
}

/* Mhtcl_Close --

	Ends a thread and frees the thread's structure.

   Arguments:

	thread -	Thread descriptor.
	objPtrVar -	If not NULL, a pointer to a variable that will
			hold the result.

   Results:

        Closes  the thread  and retrieves  the digest  data;  the result
        object will have reference counter  set to zero.  If "objPtr" is
        NULL:  the  digest  is  discarded.   Returns NULL  or  an  error
        descriptor; in  case of  error the thread  is not close:  we can
        invoke this function again later.

	  Inspection of "mhash_hmac_deinit()"  from "lib/mhash.c" in the
	source code  of MHASH version 0.8.18, reveals  that the function
	can fail only if memory allocation fails.

*/

More_Error
Mhtcl_Close (Mhtcl_Thread thread, Tcl_Obj **objPtrVar)
{
  Tcl_Obj *		objPtr;
  unsigned char *	bufPtr;
  int			e;
  More_Error		error;


  debug("closing thread %p", thread);

  objPtr = Tcl_NewByteArrayObj(NULL, 0);
  bufPtr = Tcl_SetByteArrayLength(objPtr, thread->result_length);

  if (HASH_THREAD == thread->type)
    {
      mhash_deinit(thread->data, bufPtr);
    }
  else
    {
      e = mhash_hmac_deinit(thread->data, bufPtr);
      if (-1 == e)
	{
	  Tcl_IncrRefCount(objPtr);
	  Tcl_DecrRefCount(objPtr);
	  error = More_ErrorNoMemory();
	  More_ErrorSetInt(error, ENOMEM);
	  return error;
	}
    }

  ckfree((char *) thread);
  *objPtrVar = objPtr;
  return NULL;
}

/* Mhtcl_Copy --

	Duplicates a thread.

   Arguments:

	src -		The source thread descriptor.
	dstVar -	The variable that will hold the target
			thread descriptor.

   Results:

        Returns NULL or an error descriptor.

	  Inspection  of "mhash_cp()" from  "lib/mhash.c" in  the source
	code of MHASH version 0.8.18) reveals that the function can fail
	only if memory allocation fails.

*/

More_Error
Mhtcl_Copy (Mhtcl_Thread src, Mhtcl_Thread *dstVar)
{
  More_Error	error;
  Mhtcl_Thread	dst;
  MHASH		data;

  
  debug("entering thread copy, source thread %p", src);
  debug("thread algorithm %d, result length %d",
	src->algorithm, src->result_length);

  data = mhash_cp(src->data);
  if (MHASH_FAILED == data)
    {
      error = More_ErrorNoMemory();
      More_ErrorSetInt(error, ENOMEM);
      return error;
    }

  assert(memcmp(((mhash_instance_t *)src->data)->state,
		((mhash_instance_t *)data)->state,
		((mhash_instance_t *) src->data)->state_size) == 0);

  dst = (Mhtcl_Thread) ckalloc(sizeof(Thread));
  memcpy(dst, src, sizeof(Thread));
  dst->data = data;

  *dstVar = dst;
  debug("successfully duplicated thread: %p", *dstVar);
  return NULL;
}

/* Mhtcl_Abort --

	Frees all the resources associated to a thread.

   Arguments:

	thread -	Thread descriptor.

   Results:

        Returns nothing.

   Bugs:

	In "tiger160_digest()" from  "lib/tiger.c" in the MHASH sources,
	version  0.8.18,  there  is  no  check for  NULL  digest  buffer
	pointer: this causes a segmentation fault if "mhash_deinit()" is
	used with NULL as second parameter to abort a thread.

	  "Mhtcl_Abort()"   circumvents  this   testing   the  algorithm
	before.  The usage of  "mhash_end()" ans  "mhash_hmac_end()" can
	lead to a  memory leakage if the invocation  to malloc that they
	perform  fails:  "Mhtcl_Abort()"  frees  only the  MHTCL  thread
	structure leaving alone the MHASH one.

*/

void
Mhtcl_Abort (Mhtcl_Thread thread)
{
  int			e;
  unsigned char *	buffer;


  debug("aborting thread %p", thread);

  if (HASH_THREAD == thread->type)
    {
      debug("aborting hash thread %p, %s", thread,
	    HashAlgos[thread->algorithm].name);

      if ((MHTCL_HASH_TIGER160 != thread->algorithm) &&
	  (MHTCL_HASH_WHIRLPOOL != thread->algorithm))
	{
	  mhash_deinit(thread->data, NULL);
	}
      else
	{
	  buffer = mhash_end(thread->data);
	  if (buffer)
	    {
	      free(buffer);
	    }
	}
    }
  else
    {
      debug("aborting HMAC thread %p", thread);

      if ((MHTCL_HASH_TIGER160 != thread->algorithm) &&
	  (MHTCL_HASH_WHIRLPOOL != thread->algorithm))
	{
	  e = mhash_hmac_deinit(thread->data, NULL);
	  if (-1 == e)
	    {
	      debug("error freeing data");
	      free(thread->data);
	    }
	}
      else
	{
	  buffer = mhash_hmac_end(thread->data);
	  if (buffer)
	    {
	      free(buffer);
	    }
	}
    }

  debug("releasing resources");
  ckfree((char *) thread);
}

/* Mhtcl_Keygen --

	Generates a keyword from a passphrase.

   Arguments:

	algorithm -	An integer identifying the key generation algorithm.
	length -	The requested length of the key.
	password -	The password block.
	hash1 -		An integer selecting a hash algorithm that
			may be used by the key generation algorithm.
	hash2 -		An integer selecting another hash algorithm
			that may be used by the key generation algorithm.
        count -		If used by the selected key generation algorithm,
			it's the  number of bytes to hash repeatedly.
	salt -		The salt block: an optional salt that may be used
			by the key generation algorithm; may be an empty
			block.
	keyVar -	Pointer to the variable that will hold a
			Byte Array object holding the key.

   Results:

	Generates a key. Returns NULL or an error descriptor.

*/

More_Error
Mhtcl_Keygen (algorithm, length, password, hash1, hash2, count, salt, keyVar)
     int	algorithm;
     int	length;
     More_Block	password;
     int	hash1;
     int	hash2;
     int	count;
     More_Block salt;
     Tcl_Obj **	keyVar;
{
  int			e;
  unsigned char *	keyBuf;
  KEYGEN		data;
  Tcl_Obj *		keyObj;
  CONST char *		errorString;
  More_Error		error;


  debug("enter key generation, algorithm %d", algorithm);

  if (NotKeygenAlgo)
    {
      errorString = "invalid algorithm selector";
      goto Logic_Error;
    }
  debug("selected algorithm: %s", KeygenAlgos[algorithm].name);

  if (length <= 0)
    {
      errorString = "requested null key";
      goto Logic_Error;
    }

  if ((MHTCL_KEYGEN_HEX == algorithm) && (length * 2 != password.len))
    {
      errorString = "when using the hex algorithm the requested key length must be half of the password length";
      goto Logic_Error;
    }

  debug("password length: %d, salt length %d, count %d, hashes: %d, %d",
	password.len, salt.len, count, hash1, hash2);

  /* Initialise the structure. */

  data.hash_algorithm[0] = (hash1 != MHTCL_HASH_NONE)? HashAlgos[hash1].id : 0;
  data.hash_algorithm[1] = (hash2 != MHTCL_HASH_NONE)? HashAlgos[hash2].id : 0;
  data.count		= count;
  data.salt		= salt.ptr;
  data.salt_size	= salt.len;

  /*
    Initialise the key object.
  */

  keyObj = Tcl_NewByteArrayObj(NULL, 0);
  Tcl_SetByteArrayLength(keyObj, length);
  keyBuf = Tcl_GetByteArrayFromObj(keyObj, &length);

  /* Generate the key. */

  debug("generating key, requested length: %d", length);
  e = mhash_keygen_ext(KeygenAlgos[algorithm].id, data,
		       keyBuf, length, (void *) password.ptr, password.len);
  if (e < 0)
    {
      Tcl_IncrRefCount(keyObj);
      Tcl_DecrRefCount(keyObj);
      errorString = "not enough memory";
      goto Runtime_Error;
    }

  *keyVar = keyObj;
  return NULL;

 Logic_Error:
  error = More_ErrorNew();
  More_ErrorLogicStr(error, errorString);
  return error;

 Runtime_Error:
  error = More_ErrorNew();
  More_ErrorRuntimeStr(error, errorString);
  return error;
}

/* Mhtcl_KeygenExists --

	Determines if an algorithm is included in the library.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns true if the algorithm is present, else false.

*/

int
Mhtcl_KeygenExists (int algorithm)
{
  return (NotKeygenAlgo)? 0 : \
    ((mhash_get_keygen_name_static(KeygenAlgos[algorithm].id))? 1 : 0);
}

/* Mhtcl_HashExists --

	Determines if an algorithm is included in the library.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns true if the algorithm is present, else false.

*/

int
Mhtcl_HashExists (int algorithm)
{
  return (NotHashAlgo)? 0 : \
    ((mhash_get_hash_name_static(HashAlgos[algorithm].id))? 1 : 0);
}

/* Mhtcl_MhashVersion --

	Returns a number identifying the MHASH C API version.

   Arguments:

	None.

   Results:

        Returns the number.

*/

int
Mhtcl_MhashVersion (void)
{
  return MHASH_API_VERSION;
}

/* Mhtcl_IsHMACAlgo --

	Determines if a hash algorithm can be used for HMAC.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns true if the algorithm can be used for HMAC, else returns
        false.

*/

int
Mhtcl_IsHMACAlgo (int algorithm)
{
  int	pblock;


  if (NotHashAlgo)
    {
      return 0;
    }
  pblock = mhash_get_hash_pblock(HashAlgos[algorithm].id);
  return (pblock)? 1 : 0;
}

/* Mhtcl_HashBlockSize --

	Accesses  the  size of  the  block  used  by the  selected  hash
	algorithm.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns the block size..

   Notes:

        The value  provided by this function  is the size  of the output
        digest produced by a hash algorithm.

*/

size_t
Mhtcl_HashBlockSize (int algorithm)
{
  return mhash_get_block_size(HashAlgos[algorithm].id);
}

/* Mhtcl_KeygenUsesSalt --

	Checks if a key generation algorithm needs a salt.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns true if the algorithm needs a salt.

*/

int
Mhtcl_KeygenUsesSalt (int algorithm)
{
  return mhash_keygen_uses_salt(KeygenAlgos[algorithm].id);
}


/* Mhtcl_KeygenUsesCount --

	Checks if a  key generation algorithm uses the  "count" field of
	the  configuration structure,  hashing  repeatedly the  password
	until "count" bytes have been processed.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns true or false.

*/

int
Mhtcl_KeygenUsesCount (int algorithm)
{
  return mhash_keygen_uses_count(KeygenAlgos[algorithm].id);
}


/* Mhtcl_KeygenSaltSize --

	Accesses  the size  of the  salt  required by  a key  generation
	algorithm.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns  the salt  size or  zero if  no limit  to the  salt size
        exists.

*/

int
Mhtcl_KeygenSaltSize (int algorithm)
{
  return mhash_get_keygen_salt_size (KeygenAlgos[algorithm].id);
}

/* Mhtcl_KeygenMaxKeySize --

	Accesses the max size of the key produced by an algorithm.

   Arguments:

	algorithm -	Algorithm selector.

   Results:

        Returns the max size or zero if there's no limit to the size for
        the selected algorithm.

*/

int
Mhtcl_KeygenMaxKeySize (int algorithm)
{
  return mhash_get_keygen_max_key_size (KeygenAlgos[algorithm].id);
}

/* Mhtcl_KeygenUsesHash --

	Checks  if  a  key   generation  algorithm  makes  use  of  hash
	algorithms.

   Arguments:

	algorithm -	the algorithm identifier

   Results:

        Returns the number of hash algorithms used (max 2) or zero if no
        hash algorithms are used.

*/

int
Mhtcl_KeygenUsesHash (int algorithm)
{
  return mhash_keygen_uses_hash_algorithm (KeygenAlgos[algorithm].id);
}

/* Mhtcl_ListHashAlgos --

	Builds a list of hash algorithm names.

   Arguments:

	None.

   Results:

        Returns the pointer to the list object.

*/

Tcl_Obj *
Mhtcl_ListHashAlgos (void)
{
  Tcl_Obj *	objPtr;
  int		i;


  objPtr = Tcl_NewListObj(0, NULL);
  for (i=0; i<MHTCL_HASH_INVALID; ++i)
    {
      Tcl_ListObjAppendElement(NULL, objPtr,
			       Tcl_NewStringObj(HashAlgos[i].name, -1));
    }
  return objPtr;
}

/* Mhtcl_ListKeygenAlgos --

	Builds a list of key generation algorithm names.

   Arguments:

	None.

   Results:

        Returns the pointer to the list object.

*/

Tcl_Obj *
Mhtcl_ListKeygenAlgos (void)
{
  Tcl_Obj *	objPtr;
  int		i;


  objPtr = Tcl_NewListObj(0, NULL);
  for (i=0; i<MHTCL_KEYGEN_INVALID; ++i)
    {
      Tcl_ListObjAppendElement(NULL, objPtr,
			       Tcl_NewStringObj(KeygenAlgos[i].name, -1));
    }
  return objPtr;
}

/* Mhtcl_GetHashAlgFromObj --

	Extracts  from  an  object,   an  integer  representing  a  hash
	algorithm.   The object  must  hold  a string  from  the set  of
	recognised algorithm names.

	  This function can fail if the string is not an algorithm name.

   Arguments:

	interp -	If not NULL, the interpreter to use to report
			errors.
	objPtr -	Pointer to the source object.
	algPtr -	Pointer to an integer variable that will hold
			the algorithm index.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
Mhtcl_GetHashAlgFromObj (interp, objPtr, algPtr)
     Tcl_Interp *	interp;
     Tcl_Obj *		objPtr;
     int *		algPtr;
{
  int e, index;


  e = Tcl_GetIndexFromObjStruct(interp, objPtr, HashAlgos,
				sizeof(HashSelector), "algorithm",
				TCL_EXACT, &index);
  if (TCL_OK != e)
    {
      return e;
    }

  *algPtr = index;
  return TCL_OK;
}

/* Mhtcl_GetKeygenAlgFromObj --

	Extracts from an object, an integer representing a key generator
	algorithm.   The object  must  hold  a string  from  the set  of
	recognised algorithm names.

	  This function can fail if the string is not an algorithm name.

   Arguments:

	interp -	If not NULL, the interpreter to use to report
			errors.
	objPtr -	Pointer to the source object.
	algPtr -	Pointer to an integer variable that will hold
			the algorithm index.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
Mhtcl_GetKeygenAlgFromObj (interp, objPtr, algPtr)
     Tcl_Interp *	interp;
     Tcl_Obj *		objPtr;
     int *		algPtr;
{
  int e, index;


  e = Tcl_GetIndexFromObjStruct(interp, objPtr, KeygenAlgos,
				sizeof(KeygenSelector),
				"algorithm", TCL_EXACT, &index);
  if (TCL_OK == e)
    {
      *algPtr = index;
      return TCL_OK;
    }
  else
    {
      return e;
    }
}

/* end of file */
