# tests.tcl --
# 
# Part of: MHTCL
# Contents: real tests for the library
# Date: Sun Aug 10, 2003
# 
# Abstract
# 
#	These are the real tests; this file is sourced by both
#	"mhash.test" and "stubs.test".
# 
#	  Note: some keygen tests requires hash algorithm selection, if
#	the test does not make it, it expects the defaults to be: hash1
#	md5 and hash2 sha1.
#
# Copyright (c) 2003, 2004 Marco Maggi
# 
# This is free software; you  can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software  Foundation; either version  2.1 of the License,  or (at
# your option) any later version.
# 
# This library  is distributed in the  hope that it will  be useful, but
# WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# Lesser General Public License for more details.
# 
# You  should have  received a  copy of  the GNU  Lesser  General Public
# License along  with this library; if  not, write to  the Free Software
# Foundation, Inc.,  59 Temple Place,  Suite 330, Boston,  MA 02111-1307
# USA
# 

#PAGE
## ------------------------------------------------------------
## Test namespace.
## ------------------------------------------------------------

::tcltest::configure -verbose "pass error start"

namespace eval test {

    variable	forever 0
    variable	debugging 0
    variable	ns [namespace current]
    variable	fileName [file join [::tcltest::configure -testdir] proof.txt]
    
    
    proc debug { string } {
	variable	debugging
	if { $debugging } {
	    puts [format "%-25s--- %s" test $string]
	    flush stdout
	}
    }
    
    proc hash { data algo } {
	variable	mhash

	set thread [$mhash hash $algo]
	debug "processing data"
	puts -nonewline $thread $data
	debug "making digest"
	set digest [fconfigure $thread -digest]
	close $thread

	binary scan $digest H[expr {[string length $digest]*2}] digest
	return [string toupper $digest]
    }

    proc hmac { algo passphrase data } {
	variable	mhash

	set thread [$mhash hmac $algo $passphrase]
	puts -nonewline $thread $data
	set digest [fconfigure $thread -digest]
	close $thread

	binary scan $digest H[expr {[string length $digest]*2}] digest
	return [string toupper $digest]
    }

    proc keygen { algo password args } {
	variable	mhash
	set length 10
	set data [eval {$mhash keygen $algo $password $length} $args]

	binary scan $data H[expr {2*[string length $data]}] data
	return [string toupper $data]
    }

    proc hashFile { algorithm data } {
	variable	fileName
	variable	ns

	::tcltest::makeFile $data $fileName
	set thread	[mhash hash $algorithm]
	set channel	[open $fileName RDONLY]
	fconfigure $thread -translation binary
	fconfigure $channel -translation binary
	puts -nonewline $thread [read $channel]
	set digest [fconfigure $thread -digest]
	close $channel
	close $thread
	deleteFile

	binary scan $digest H[expr {[string length $digest]*2}] digest
	return [string toupper $digest]
    }

    proc fcopyHashFile { algorithm data } {
	variable	fileName
	variable	ns

	::tcltest::makeFile $data $fileName
	set thread	[mhash hash $algorithm]
	set channel	[open $fileName RDONLY]
	fconfigure $thread -translation binary
	fconfigure $channel -translation binary
	fcopy $channel $thread \
	    -command [list ${ns}::fcopyCompleteCallback $channel $thread]
	vwait ${ns}::forever
	set digest [fconfigure $thread -digest]
	close $channel
	close $thread
	deleteFile

	binary scan $digest H[expr {[string length $digest]*2}] digest
	return [string toupper $digest]
    }
    
    proc fcopyCompleteCallback { channel thread bytes {error {}} } {
	variable	ns
	
	debug "fcopy callback: $bytes bytes, eof [eof $channel]"
	set ${ns}::forever 1
	return
    }
    
    proc deleteFile { } {
	variable	fileName
	::tcltest::removeFile $fileName
    }
    
#PAGE

::tcltest::test hash-1.1 "crc32" {
    -body	{
	hash {} "crc32"
    }
    -result 00000000
}

::tcltest::test hash-1.2 "crc32" {
    -body	{
	hash a "crc32"
    }
    -result 6B9B9319
}

::tcltest::test hash-1.3 "crc32" {
    -body	{
	hash abc "crc32"
    }
    -result 73BB8C64
}

::tcltest::test hash-1.4 "crc32" {
    -body	{
	hash checksum "crc32"
    }
    -result 7FBEB02E
}

::tcltest::test hash-1.5 "crc32" {
    -body	{
	hash "check the diagonal" "crc32"
    }
    -result 34AD9A70

}

#PAGE

::tcltest::test hash-2.1 "crc32b" {
    -body	{
	hash {} "crc32b"
    }
    -result 00000000
}

::tcltest::test hash-2.2 "crc32b" {
    -body	{
	hash a "crc32b"
    }
    -result 43BEB7E8
}

::tcltest::test hash-2.3 "crc32b" {
    -body	{
	hash abc "crc32b"
    }
    -result C2412435
}

::tcltest::test hash-2.4 "crc32b" {
    -body	{
	hash checksum "crc32b"
    }
    -result 9ADF6FDE
}

::tcltest::test hash-2.5 "crc32b" {
    -body	{
	hash "check the diagonal" "crc32b"
    }
    -result 8B64DC5C
}

#PAGE

::tcltest::test hash-3.1 "md5" {
    -body	{
	hash {} md5
    }
    -result D41D8CD98F00B204E9800998ECF8427E
}

::tcltest::test hash-3.2 "md5" {
    -body	{
	hash a md5
    }
    -result 0CC175B9C0F1B6A831C399E269772661
}

::tcltest::test hash-3.3 "md5" {
    -body	{
	hash abc md5
    }
    -result 900150983CD24FB0D6963F7D28E17F72
}

::tcltest::test hash-3.4 "md5" {
    -body	{
	hash "message digest" md5
    }
    -result F96B697D7CB7938D525A2F31AAF161D0
}

::tcltest::test hash-3.5 "md5" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz md5
    }
    -result C3FCD3D76192E4007DFB496CCA67E13B
}

::tcltest::test hash-3.6 "md5" {
    -body	{
	hash \
		ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 \
		md5
    }
    -result D174AB98D277D9F5A5611C2C9F419D9F
}

::tcltest::test hash-3.7 "md5" {
    -body	{
	hash 12345678901234567890123456789012345678901234567890123456789012345678901234567890 md5
    }
    -result 57EDF4A22BE3C955AC49DA2E2107B67A
}

#PAGE

::tcltest::test hash-4.1 "sha1" {
    -body	{
	hash {} sha1
    }
    -result DA39A3EE5E6B4B0D3255BFEF95601890AFD80709
}

::tcltest::test hash-4.2 "sha1" {
    -body	{
	hash a sha1
    }
    -result 86F7E437FAA5A7FCE15D1DDCB9EAEAEA377667B8
}

::tcltest::test hash-4.3 "sha1" {
    -body	{
	hash abc sha1
    }
    -result A9993E364706816ABA3E25717850C26C9CD0D89D
}

::tcltest::test hash-4.4 "sha1" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq sha1
    }
    -result 84983E441C3BD26EBAAE4AA1F95129E5E54670F1
}

::tcltest::test hash-4.5 "sha1" {
    -body	{
	hash "check the diagonal" sha1
    }
    -result 2DB3133909E17F418C6426C0E60313DAB99C7C32

}

#PAGE

::tcltest::test hash-5.1 "haval256" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz haval256
    }
    -result 72FAD4BDE1DA8C8332FB60561A780E7F504F21547B98686824FC33FC796AFA76
}

::tcltest::test hash-5.2 "haval256" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 haval256
    }
    -result 899397D96489281E9E76D5E65ABAB751F312E06C06C07C9C1D42ABD31BB6A404
}

::tcltest::test hash-5.3 "haval224" {
    -body	{
	hash "0123456789" haval224
    }
    -result EE345C97A58190BF0F38BF7CE890231AA5FCF9862BF8E7BEBBF76789
}

::tcltest::test hash-5.4 "haval192" {
    -body	{
	hash "HAVAL" haval192
    }
    -result 8DA26DDAB4317B392B22B638998FE65B0FBE4610D345CF89
}

::tcltest::test hash-5.5 "haval160" {
    -body	{
	hash a haval160
    }
    -result 4DA08F514A7275DBC4CECE4A347385983983A830
}

::tcltest::test hash-5.6 "haval128" {
    -body	{
	hash {} haval128
    }
    -result C68F39913F901F3DDF44C707357A7D70
}

#PAGE

::tcltest::test hash-6.1 "ripemd160" {
    -body	{
	hash {} ripemd160
    }
    -result 9C1185A5C5E9FC54612808977EE8F548B2258D31
}

::tcltest::test hash-6.2 "ripemd160" {
    -body	{
	hash a ripemd160
    }
    -result 0BDC9D2D256B3EE9DAAE347BE6F4DC835A467FFE
}

::tcltest::test hash-6.3 "ripemd160" {
    -body	{
	hash abc ripemd160
    }
    -result 8EB208F7E05D987A9B044A8E98C6B087F15A0BFC
}

::tcltest::test hash-6.4 "ripemd160" {
    -body	{
	hash "message digest" ripemd160
    }
    -result  5D0689EF49D2FAE572B881B123A85FFA21595F36
}

::tcltest::test hash-6.5 "ripemd160" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz ripemd160
    }
    -result F71C27109C692C1B56BBDCEB5B9D2865B3708DBC
}

::tcltest::test hash-6.6 "ripemd160" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq ripemd160
    }
    -result 12A053384A9C0C88E405A06C27DCF49ADA62EB2B
}

::tcltest::test hash-6.7 "ripemd160" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 ripemd160
    }
    -result B0E20B6E3116640286ED3A87A5713079B21F5189
}

::tcltest::test hash-6.8 "ripemd128" {
    -body	{
	hash {} ripemd128
    }
    -result CDF26213A150DC3ECB610F18F6B38B46
}

::tcltest::test hash-6.9 "ripemd128" {
    -body	{
	hash a ripemd128
    }
    -result 86BE7AFA339D0FC7CFC785E72F578D33
}

::tcltest::test hash-6.10 "ripemd128" {
    -body	{
	hash abc ripemd128
    }
    -result C14A12199C66E4BA84636B0F69144C77
}

::tcltest::test hash-6.11 "ripemd128" {
    -body	{
	hash "message digest" ripemd128
    }
    -result 9E327B3D6E523062AFC1132D7DF9D1B8
}

::tcltest::test hash-6.12 "ripemd128" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz ripemd128
    }
    -result FD2AA607F71DC8F510714922B371834E
}

::tcltest::test hash-6.13 "ripemd128" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq ripemd128
    }
    -result A1AA0689D0FAFA2DDC22E88B49133A06
}
		
::tcltest::test hash-6.14 "ripemd128" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 ripemd128
    }
    -result D1E959EB179C911FAEA4624C60C5C702
}
		
::tcltest::test hash-6.15 "ripemd128" {
    -body	{
	hash 12345678901234567890123456789012345678901234567890123456789012345678901234567890 ripemd128
    }
    -result 3F45EF194732C2DBB2C4A2C769795FA3
}

::tcltest::test hash-6.16 "ripemd256" {
    -body	{
	hash {} ripemd256
    }
    -result 02BA4C4E5F8ECD1877FC52D64D30E37A2D9774FB1E5D026380AE0168E3C5522D
}

::tcltest::test hash-6.17 "ripemd256" {
    -body	{
	hash a ripemd256
    }
    -result F9333E45D857F5D90A91BAB70A1EBA0CFB1BE4B0783C9ACFCD883A9134692925
}

::tcltest::test hash-6.18 "ripemd256" {
    -body	{
	hash abc ripemd256
    }
    -result AFBD6E228B9D8CBBCEF5CA2D03E6DBA10AC0BC7DCBE4680E1E42D2E975459B65
}

::tcltest::test hash-6.19 "ripemd256" {
    -body	{
	hash "message digest" ripemd256
    }
    -result 87E971759A1CE47A514D5C914C392C9018C7C46BC14465554AFCDF54A5070C0E
}

::tcltest::test hash-6.20 "ripemd256" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz ripemd256
    }
    -result 649D3034751EA216776BF9A18ACC81BC7896118A5197968782DD1FD97D8D5133
}

::tcltest::test hash-6.21 "ripemd256" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq ripemd256
    }
    -result 3843045583AAC6C8C8D9128573E7A9809AFB2A0F34CCC36EA9E72F16F6368E3F
}
		
::tcltest::test hash-6.22 "ripemd256" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 ripemd256
    }
    -result 5740A408AC16B720B84424AE931CBB1FE363D1D0BF4017F1A89F7EA6DE77A0B8
}
		
::tcltest::test hash-6.23 "ripemd256" {
    -body	{
	hash 12345678901234567890123456789012345678901234567890123456789012345678901234567890 ripemd256
    }
    -result 06FDCC7A409548AAF91368C06A6275B553E3F099BF0EA4EDFD6778DF89A890DD
}
		
::tcltest::test hash-6.24 "ripemd256" {
    -body	{
	# `perl -e 'print "a"x1000000'` 
	hash [string repeat a 1000000] ripemd256
    }
    -result AC953744E10E31514C150D4D8D7B677342E33399788296E43AE4850CE4F97978
}

::tcltest::test hash-6.25 "ripemd320" {
    -body	{
	hash "" ripemd320
    }
    -result 22D65D5661536CDC75C1FDF5C6DE7B41B9F27325EBC61E8557177D705A0EC880151C3A32A00899B8
}

::tcltest::test hash-6.26 "ripemd320" {
    -body	{
	hash a ripemd320
    }
    -result CE78850638F92658A5A585097579926DDA667A5716562CFCF6FBE77F63542F99B04705D6970DFF5D
}

::tcltest::test hash-6.27 "ripemd320" {
    -body	{
	hash abc ripemd320
    }
    -result DE4C01B3054F8930A79D09AE738E92301E5A17085BEFFDC1B8D116713E74F82FA942D64CDBC4682D
}

::tcltest::test hash-6.28 "ripemd320" {
    -body	{
	hash "message digest" ripemd320
    }
    -result 3A8E28502ED45D422F68844F9DD316E7B98533FA3F2A91D29F84D425C88D6B4EFF727DF66A7C0197
}

::tcltest::test hash-6.29 "ripemd320" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz ripemd320
    }
    -result CABDB1810B92470A2093AA6BCE05952C28348CF43FF60841975166BB40ED234004B8824463E6B009
}

::tcltest::test hash-6.30 "ripemd320" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq ripemd320
    }
    -result D034A7950CF722021BA4B84DF769A5DE2060E259DF4C9BB4A4268C0E935BBC7470A969C9D072A1AC
}

::tcltest::test hash-6.31 "ripemd320" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 ripemd320
    }
    -result ED544940C86D67F250D232C30B7B3E5770E0C60C8CB9A4CAFE3B11388AF9920E1B99230B843C86A4
}

::tcltest::test hash-6.32 "ripemd320" {
    -body	{
	hash 12345678901234567890123456789012345678901234567890123456789012345678901234567890 ripemd320
    }
    -result 557888AF5F6D8ED62AB66945C6D2A0A47ECD5341E915EB8FEA1D0524955F825DC717E4A008AB2D42
}

::tcltest::test hash-6.33 "ripemd320" {
    -body	{
	hash [string repeat a 1000000] ripemd320
    }
    -result BDEE37F4371E20646B8B0D862DDA16292AE36F40965E8C8509E63D1DBDDECC503E2B63EB9245BB66
}

#PAGE

::tcltest::test hash-7.1 "tiger" {
    -body	{
	hash {} tiger
    }
    -result 24F0130C63AC933216166E76B1BB925FF373DE2D49584E7A
}

::tcltest::test hash-7.2 "tiger" {
    -body	{
	hash abc tiger
    }
    -result F258C1E88414AB2A527AB541FFC5B8BF935F7B951C132951
}

::tcltest::test hash-7.3 "tiger" {
    -body	{
	hash Tiger tiger
    }
    -result 9F00F599072300DD276ABB38C8EB6DEC37790C116F9D2BDF
}

::tcltest::test hash-7.4 "tiger" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+- tiger
    }
    -result 87FB2A9083851CF7470D2CF810E6DF9EB586445034A5A386
}

::tcltest::test hash-7.5 "tiger" {
    -body	{
	hash "ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz+0123456789" tiger
    }
    -result 467DB80863EBCE488DF1CD1261655DE957896565975F9197
}

::tcltest::test hash-7.6 "tiger" {
    -body	{
	hash "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham" tiger
    }
    -result 0C410A042968868A1671DA5A3FD29A725EC1E457D3CDB303
}

::tcltest::test hash-7.7 "tiger" {
    -body	{
	hash "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge." tiger
    }
    -result EBF591D5AFA655CE7F22894FF87F54AC89C811B6B0DA3193
}

::tcltest::test hash-7.8 "tiger" {
    -body	{
	hash "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-" tiger
    }
    -result 00B83EB4E53440C576AC6AAEE0A7485825FD15E70A59FFE4
}

::tcltest::test hash-7.9 "tiger160" {
    -body	{
	hash {} tiger160
    }
    -result 24F0130C63AC933216166E76B1BB925FF373DE2D
}

::tcltest::test hash-7.10 "tiger160" {
    -body	{
	hash abc tiger160
    }
    -result F258C1E88414AB2A527AB541FFC5B8BF935F7B95
}

::tcltest::test hash-7.11 "tiger160" {
    -body	{
	hash Tiger tiger160
    }
    -result 9F00F599072300DD276ABB38C8EB6DEC37790C11
}

::tcltest::test hash-7.12 "tiger160" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+- tiger160
    }
    -result 87FB2A9083851CF7470D2CF810E6DF9EB5864450
}

::tcltest::test hash-7.13 "tiger160" {
    -body	{
	hash "ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz+0123456789" tiger160
    }
    -result 467DB80863EBCE488DF1CD1261655DE957896565
}

::tcltest::test hash-7.14 "tiger128" {
    -body	{
	hash {} tiger128
    }
    -result 24F0130C63AC933216166E76B1BB925F
}

::tcltest::test hash-7.15 "tiger128" {
    -body	{
	hash abc tiger128
    }
    -result F258C1E88414AB2A527AB541FFC5B8BF
}

::tcltest::test hash-7.16 "tiger128" {
    -body	{
	hash Tiger tiger128
    }
    -result 9F00F599072300DD276ABB38C8EB6DEC
}

#PAGE

::tcltest::test hash-8.1 "gost" {
    -body	{
	hash {} gost
    }
    -result CE85B99CC46752FFFEE35CAB9A7B0278ABB4C2D2055CFF685AF4912C49490F8D
}

::tcltest::test hash-8.2 "gost" {
    -body	{
	hash a gost
    }
    -result D42C539E367C66E9C88A801F6649349C21871B4344C6A573F849FDCE62F314DD
}

::tcltest::test hash-8.3 "gost" {
    -body	{
	hash abc gost
    }
    -result F3134348C44FB1B2A277729E2285EBB5CB5E0F29C975BC753B70497C06A4D51D
}

::tcltest::test hash-8.4 "gost" {
    -body	{
	hash "This is message, length=32 bytes" gost
    }
    -result B1C466D37519B82E8319819FF32595E047A28CB6F83EFF1C6916A815A637FFFA
}

::tcltest::test hash-8.5 "gost" {
    -body	{
	hash "Suppose the original message has length = 50 bytes" gost
    }
    -result 471ABA57A60A770D3A76130635C1FBEA4EF14DE51F78B4AE57DD893B62F55208
}

#PAGE

::tcltest::test hash-9.1 "md4" {
    -body	{
	hash {} md4
    }
    -result 31D6CFE0D16AE931B73C59D7E0C089C0
}

::tcltest::test hash-9.2 "md4" {
    -body	{
	hash a md4
    }
    -result BDE52CB31DE33E46245E05FBDBD6FB24
}

::tcltest::test hash-9.3 "md4" {
    -body	{
	hash abc md4
    }
    -result A448017AAF21D8525FC10AE87AA6729D
}

::tcltest::test hash-9.3 "md4" {
    -body	{
	hash "message digest"  md4
    }
    -result D9130A8164549FE818874806E1C7014B
}

::tcltest::test hash-9.4 "md4" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz md4
    }
    -result D79E1C308AA5BBCDEEA8ED63DF412DA9
}

::tcltest::test hash-9.5 "md4" {
    -body	{
	hash 12345678901234567890123456789012345678901234567890123456789012345678901234567890 md4
    }
    -result E33B4DDC9C38F2199C3E7B164FCC0536
}

#PAGE

::tcltest::test hash-10.1 "sha256" {
    -body	{
	hash {} sha256
    }
    -result E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855
}

::tcltest::test hash-10.2 "sha256" {
    -body	{
	hash a sha256
    }
    -result CA978112CA1BBDCAFAC231B39A23DC4DA786EFF8147C4E72B9807785AFEE48BB
}

::tcltest::test hash-10.3 "sha256" {
    -body	{
	hash abc sha256
    }
    -result BA7816BF8F01CFEA414140DE5DAE2223B00361A396177A9CB410FF61F20015AD
}

::tcltest::test hash-10.4 "sha256" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq sha256
    }
    -result 248D6A61D20638B8E5C026930C3E6039A33CE45964FF2167F6ECEDD419DB06C1
}

::tcltest::test hash-10.5 "sha256" {
    -body	{
	hash abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu sha256
    }
    -result CF5B16A778AF8380036CE59E7B0492370B249B11E8F07A51AFAC45037AFEE9D1
}

#PAGE

::tcltest::test hash-11.1 "sha224" {
    -body	{
	hash abc sha224
    }
    -result 23097D223405D8228642A477BDA255B32AADBCE4BDA0B3F7E36C9DA7
}

::tcltest::test hash-11.2 "sha224" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq sha224
    }
    -result 75388B16512776CC5DBA5DA1FD890150B0C6455CB4F58B1952522525
}

::tcltest::test hash-11.3 "sha224" {
    -body	{
	hash [string repeat a 1000000] sha224
    }
    -result 20794655980C91D8BBB4C1EA97618A4BF03F42581948B2EE4EE7AD67
}

::tcltest::test hash-11.4 "sha512" {
    -body	{
	hash abc sha512
    }
    -result DDAF35A193617ABACC417349AE20413112E6FA4E89A97EA20A9EEEE64B55D39A2192992A274FC1A836BA3C23A3FEEBBD454D4423643CE80E2A9AC94FA54CA49F
}

::tcltest::test hash-11.5 "sha512" {
    -body	{
	hash abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu sha512
    }
    -result 8E959B75DAE313DA8CF4F72814FC143F8F7779C6EB9F7FA17299AEADB6889018501D289E4900F7E4331B99DEC4B5433AC7D329EEB6DD26545E96E55B874BE909
}

::tcltest::test hash-11.6 "sha512" {
    -body	{
	hash [string repeat a 1000000] sha512
    }
    -result E718483D0CE769644E2E42C7BC15B4638E1F98B13B2044285632A803AFA973EBDE0FF244877EA60A4CB0432CE577C31BEB009C5C2C49AA2E4EADB217AD8CC09B
}

::tcltest::test hash-11.7 "sha384" {
    -body	{
	hash abc sha384
    }
    -result CB00753F45A35E8BB5A03D699AC65007272C32AB0EDED1631A8B605A43FF5BED8086072BA1E7CC2358BAECA134C825A7
}

::tcltest::test hash-11.8 "sha384" {
    -body	{
	hash abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu sha384
    }
    -result 09330C33F71147E83D192FC782CD1B4753111B173B3B05D22FA08086E3B0F712FCC7C71A557E2DB966C3E9FA91746039
}

::tcltest::test hash-11.9 "sha384" {
    -body	{
	hash [string repeat a 1000000] sha384
    }
    -result 9D0E1809716474CB086E834E310A4A1CED149E9C00F248527972CEC5704C2A5B07B8B3DC38ECC4EBAE97DDD87F3D8985
}

#PAGE

::tcltest::test hash-12.1 "whirlpool" {
    -body	{
	hash {} whirlpool
    }
    -result 19FA61D75522A4669B44E39C1D2E1726C530232130D407F89AFEE0964997F7A73E83BE698B288FEBCF88E3E03C4F0757EA8964E59B63D93708B138CC42A66EB3
}

::tcltest::test hash-12.2 "whirlpool" {
    -body	{
	hash a whirlpool
    }
    -result 8ACA2602792AEC6F11A67206531FB7D7F0DFF59413145E6973C45001D0087B42D11BC645413AEFF63A42391A39145A591A92200D560195E53B478584FDAE231A
}

::tcltest::test hash-12.3 "whirlpool" {
    -body	{
	hash abc whirlpool
    }
    -result 4E2448A4C6F486BB16B6562C73B4020BF3043E3A731BCE721AE1B303D97E6D4C7181EEBDB6C57E277D0E34957114CBD6C797FC9D95D8B582D225292076D4EEF5
}

::tcltest::test hash-12.4 "whirlpool" {
    -body	{
	hash "message digest" whirlpool
    }
    -result 378C84A4126E2DC6E56DCC7458377AAC838D00032230F53CE1F5700C0FFB4D3B8421557659EF55C106B4B52AC5A4AAA692ED920052838F3362E86DBD37A8903E
}

::tcltest::test hash-12.5 "whirlpool" {
    -body	{
	hash abcdefghijklmnopqrstuvwxyz whirlpool
    }
    -result F1D754662636FFE92C82EBB9212A484A8D38631EAD4238F5442EE13B8054E41B08BF2A9251C30B6A0B8AAE86177AB4A6F68F673E7207865D5D9819A3DBA4EB3B
}

::tcltest::test hash-12.6 "whirlpool" {
    -body	{
	hash ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 whirlpool
    }
    -result DC37E008CF9EE69BF11F00ED9ABA26901DD7C28CDEC066CC6AF42E40F82F3A1E08EBA26629129D8FB7CB57211B9281A65517CC879D7B962142C65F5A7AF01467
}

::tcltest::test hash-12.7 "whirlpool" {
    -body	{
	hash 12345678901234567890123456789012345678901234567890123456789012345678901234567890 whirlpool
    }
    -result 466EF18BABB0154D25B9D38A6414F5C08784372BCCB204D6549C4AFADB6014294D5BD8DF2A6C44E538CD047B2681A51A2C60481E88C5A20B2C2A80CF3A9A083B
}

::tcltest::test hash-12.8 "whirlpool" {
    -body	{
	hash abcdbcdecdefdefgefghfghighijhijk whirlpool
    }
    -result 2A987EA40F917061F5D6F0A0E4644F488A7A5A52DEEE656207C562F988E95C6916BDC8031BC5BE1B7B947639FE050B56939BAAA0ADFF9AE6745B7B181C3BE3FD
}

::tcltest::test hash-12.9 "whirlpool" {
    -body	{
	hash [string repeat a 1000000] whirlpool
    }
    -result 0C99005BEB57EFF50A7CF005560DDF5D29057FD86B20BFD62DECA0F1CCEA4AF51FC15490EDDC47AF32BB2B66C34FF9AD8C6008AD677F77126953B226E4ED8B01
}

#PAGE

::tcltest::test hmac-1.1 "md5" {
    -body	{
	hmac md5 password {}
    }
    -result 23D466834F70A1D657561A5EC2C51EA0
}

::tcltest::test hmac-1.2 "md5" {
    -body	{
	hmac md5 password a
    }
    -result AF59D6B4E502F82BB64AF2B712E5A0AB
}

::tcltest::test hmac-1.3 "md5" {
    -body	{
	hmac md5 password abc
    }
    -result E99378F51AB9623F27E23842415BC8BE
}

::tcltest::test hmac-1.4 "md5" {
    -body	{
	hmac md5 password "check the diagonal"
    }
    -result 2CD5F922D10D609FD52979190F59268D
}

::tcltest::test hmac-1.5 "md5" {
    -body	{
	hmac md5 password "and there was trouble taking place..."
    }
    -result A1B16C1768D6A67BFF8F908F1CF71E2C
}

::tcltest::test hmac-1.6 "md5" {
    -body	{
	hmac md5 Jefe "what do ya want for nothing?"
    }
    -result 750C783E6AB0B503EAA86E310A5DB738
}

::tcltest::test hmac-1.7 "md5" {
    -body	{
	hmac md5 "" "Hi There"
    }
    -result 9294727A3638BB1C13F48EF8158BFC9D
}

#PAGE

::tcltest::test hmac-2.1 "md4" {
    -body	{
	hmac md4 password {}
    }
    -result AD728E8DB14ADECC49A3FC361B30B715
}

::tcltest::test hmac-2.2 "md4" {
    -body	{
	hmac md4 password a
    }
    -result D90EB4E82D23763AE4B3A8FDD35D1785
}

::tcltest::test hmac-2.3 "md4" {
    -body	{
	hmac md4 password abc
    }
    -result 3E17F93610CA3FBD3F9F6B5AC8EC8E6A
}

::tcltest::test hmac-2.4 "md4" {
    -body	{
	hmac md4 password "check the diagonal"
    }
    -result 51561826AD7C5ADEDBCB81FB22F05EDB
}

::tcltest::test hmac-2.5 "md4" {
    -body	{
	hmac md4 password "and there was trouble taking place..."
    }
    -result C39AD49CBB3351CB3FC4E63F07A1D0C5

}

#PAGE

::tcltest::test hmac-3.1 "sha1" {
    -body	{
	hmac sha1 password {}
    }
    -result F1CCDB92A929F5149ED40C5D0F3F2FBBE7809521
}

::tcltest::test hmac-3.2 "sha1" {
    -body	{
	hmac sha1 password a
    }
    -result 1AD9FF421FBEBAD5ABA4CE290B67D6CED227DCD6
}

::tcltest::test hmac-3.3 "sha1" {
    -body	{
	hmac sha1 password abc
    }
    -result 6F7FE6B8DBE7D48237C88338F5846F4A33B19B67
}

::tcltest::test hmac-3.4 "sha1" {
    -body	{
	hmac sha1 password "check the diagonal"
    }
    -result 5219EB40E205E7AD3913625C5F67CFE68FBB7FE0
}

::tcltest::test hmac-3.5 "sha1" {
    -body	{
	hmac sha1 password "and there was trouble taking place..."
    }
    -result 62CA877E3239C785BA58D7EAF281D07D151CC148
}

#PAGE

::tcltest::test hmac-4.1 "sha256" {
    -body	{
	hmac sha256 password {}
    }
    -result E56A207ACD1E6714735487C199C6F095844B7CC8E5971D86C003A7B6F36EF51E
}

::tcltest::test hmac-4.2 "sha256" {
    -body	{
	hmac sha256 password a
    }
    -result C1E2EE6783CF00EF6121511E4C2C7AAE25FC25499222562CD02DC8767046461F
}

::tcltest::test hmac-4.3 "sha256" {
    -body	{
	hmac sha256 password abc
    }
    -result FC9DC38FB7E9B9F11F016AFB34263BA7C3B74F843894E01D4F5420C5D03CC866
}

::tcltest::test hmac-4.4 "sha256" {
    -body	{
	hmac sha256 password "check the diagonal"
    }
    -result 35F2A6B550CE0431927E8434149580EFD4BFD5F84179AD00774551583BEF2497
}

::tcltest::test hmac-4.5 "sha256" {
    -body	{
	hmac sha256 password "and there was trouble taking place..."
    }
    -result 499D01917620FC7D0423024C320373C1723E871B5B2AF62082A59D38A1382C2B
}

#PAGE

::tcltest::test hmac-5.1 "tiger" {
    -body	{
	hmac tiger password {}
    }
    -result C9A51427AF5BA0E6E66FCEF15BB3A237ED74A86F5A9CC55C
}

::tcltest::test hmac-5.2 "tiger" {
    -body	{
	hmac tiger password a
    }
    -result 38449DF470EBB6E4B0EEA4368BC5E69458042A4625694559
}

::tcltest::test hmac-5.3 "tiger" {
    -body	{
	hmac tiger password abc
    }
    -result D7EEBB20512D29E4E68CC6F2CB8AE7D4F213AA5423E3625E
}

::tcltest::test hmac-5.4 "tiger" {
    -body	{
	hmac tiger password "check the diagonal"
    }
    -result B31F49F6634EB37CE0BAB8CF8DEDCE03D2D6075D8AEB9CC8
}

::tcltest::test hmac-5.5 "tiger" {
    -body	{
	hmac tiger password "and there was trouble taking place..."
    }
    -result 66B8B348B875CBEB3F98816B16C7FB7D775418317A2DBA76
}

#PAGE

::tcltest::test hmac-6.1 "tiger128" {
    -body	{
	hmac tiger128 password {}
    }
    -result EBF8F0442E166EF463A1259A02E88EB6
}

::tcltest::test hmac-6.2 "tiger128" {
    -body	{
	hmac tiger128 password a
    }
    -result F98C4054CFC1B612F835A550AE647C6F
}

::tcltest::test hmac-6.3 "tiger128" {
    -body	{
	hmac tiger128 password abc
    }
    -result FF9E0C7911203D5A8E8A1CAB37389E4C
}

::tcltest::test hmac-6.4 "tiger128" {
    -body	{
	hmac tiger128 password "check the diagonal"
    }
    -result CC49E0DCF8CDC25FC2DFD4F7BD90D311
}

::tcltest::test hmac-6.5 "tiger128" {
    -body	{
	hmac tiger128 password "and there was trouble taking place..."
    }
    -result 883D2C456EF93F3ED86539F3D27B0F09
}

#PAGE

::tcltest::test hmac-7.1 "tiger160" {
    -body	{
	hmac tiger160 password {}
    }
    -result 8BEFCA2D0184A5784203BCC8BC47A9F9EE2C0549
}

::tcltest::test hmac-7.2 "tiger160" {
    -body	{
	hmac tiger160 password a
    }
    -result C2121B6EFA05FD248E425E66685FC446C4C4BE6D
}

::tcltest::test hmac-7.3 "tiger160" {
    -body	{
	hmac tiger160 password abc
    }
    -result 28DBF6931C117C51E6F1CB125B611E38CFD418F9
}

::tcltest::test hmac-7.4 "tiger160" {
    -body	{
	hmac tiger160 password "check the diagonal"
    }
    -result 30F664E68C8ECD9C700C7209DEB311B1B495F74A
}

::tcltest::test hmac-7.5 "tiger160" {
    -body	{
	hmac tiger160 password "and there was trouble taking place..."
    }
    -result FA70E083271825E2ABDD19531306130262AF3466
}

#PAGE

::tcltest::test hmac-8.1 "ripemd160" {
    -body	{
	hmac ripemd160 password {}
    }
    -result 03EA2AE5C6875DE5EC79CE4A9A7B37837660794E
}

::tcltest::test hmac-8.2 "ripemd160" {
    -body	{
	hmac ripemd160 password a
    }
    -result 6BD339F95DB0AC781BDF512F6D6DC5069B4AE580
}

::tcltest::test hmac-8.3 "ripemd160" {
    -body	{
	hmac ripemd160 password abc
    }
    -result 901330660BE3778CB8F767BE69B8BD4D4F1A7664
}

::tcltest::test hmac-8.4 "ripemd160" {
    -body	{
	hmac ripemd160 password "check the diagonal"
    }
    -result CC690CE568CCEA8896973B03760BF8D384F53EA8
}

::tcltest::test hmac-8.5 "ripemd160" {
    -body	{
	hmac ripemd160 password "and there was trouble taking place..."
    }
    -result F8C7DCAA5B691E170ED48AD4C1ADDAABC9F0B70D
}

#PAGE

::tcltest::test hmac-9.1 "haval128" {
    -body	{
	hmac haval128 password {}
    }
    -result 4F3193D8880BF4D2954C3B3E1FB9EF04
}

::tcltest::test hmac-9.2 "haval128" {
    -body	{
	hmac haval128 password a
    }
    -result AD130BAEEFD9D3695E991DF05716C20F
}

::tcltest::test hmac-9.3 "haval128" {
    -body	{
	hmac haval128 password abc
    }
    -result E5726BC66F3E82A2A7428B615027DBC3
}

::tcltest::test hmac-9.4 "haval128" {
    -body	{
	hmac haval128 password "check the diagonal"
    }
    -result 9D42FE95AC870D3715571E046FB2DD75
}

::tcltest::test hmac-9.5 "haval128" {
    -body	{
	hmac haval128 password "and there was trouble taking place..."
    }
    -result F05CFD40DBD00350D676AD146552A21E
}

#PAGE

::tcltest::test hmac-10.1 "haval160" {
    -body	{
	hmac haval160 password {}
    }
    -result 1BEFC7283D370092CFDF14940ABCDFFC91463FAC
}

::tcltest::test hmac-10.2 "haval160" {
    -body	{
	hmac haval160 password a
    }
    -result 45DE70F2F85FF90701CF5EFA450B54C871162E2A
}

::tcltest::test hmac-10.3 "haval160" {
    -body	{
	hmac haval160 password abc
    }
    -result 570C1B99223E35B541E21E955963129B90C4072F
}

::tcltest::test hmac-10.4 "haval160" {
    -body	{
	hmac haval160 password "check the diagonal"
    }
    -result 8E67DA1F47063E07934FCD30EF0E961D825AC18D
}

::tcltest::test hmac-10.5 "haval160" {
    -body	{
	hmac haval160 password "and there was trouble taking place..."
    }
    -result 49A8C00140C2D06E6BAD479767571BF9C0013BFF
}

#PAGE

::tcltest::test hmac-11.1 "haval192" {
    -body	{
	hmac haval192 password {}
    }
    -result 4EC5254E8F7179501820EE4DDE63C6AADC042A2EABEA1926
}

::tcltest::test hmac-11.2 "haval192" {
    -body	{
	hmac haval192 password a
    }
    -result F990FB7ACB1F3BD4C10BBF9EA9BC5F91C706C537B3B66E75
}

::tcltest::test hmac-11.3 "haval192" {
    -body	{
	hmac haval192 password abc
    }
    -result 8F5486BF0985B657AF6EF12726A1AD3329C7DEA695DEFE5C
}

::tcltest::test hmac-11.4 "haval192" {
    -body	{
	hmac haval192 password "check the diagonal"
    }
    -result 9F2C4476250F66A8319C1E03F989D0567D827328FB8BB97F
}

::tcltest::test hmac-11.5 "haval192" {
    -body	{
	hmac haval192 password "and there was trouble taking place..."
    }
    -result 4DBB85A409EBD34717AFFA17287ED084F0B7E77AB6C82BE2
}

#PAGE

::tcltest::test hmac-12.1 "haval224" {
    -body	{
	hmac haval224 password {}
    }
    -result 4453C03D6E429787A1F3501530D5276E1BB574B2E7DDB7345AB1A41C
}

::tcltest::test hmac-12.2 "haval224" {
    -body	{
	hmac haval224 password a
    }
    -result 603C0425D4DEEDD35CF61B4855C720BA9B92409FFFF62FA592D34E77
}

::tcltest::test hmac-12.3 "haval224" {
    -body	{
	hmac haval224 password abc
    }
    -result 2C506057F0FE48CD4A4160368530CE5F214514052449B1F9A898D9D8
}

::tcltest::test hmac-12.4 "haval224" {
    -body	{
	hmac haval224 password "check the diagonal"
    }
    -result EE65E6E46B7A509E4DF238126A1D35582C08AC7D44554B87EB971152
}

::tcltest::test hmac-12.5 "haval224" {
    -body	{
	hmac haval224 password "and there was trouble taking place..."
    }
    -result 78D53DE7FDDA38AC41E81B5649BEECC71146D314B32E7BE594AA40EE

}

#PAGE

::tcltest::test hmac-13.1 "haval256" {
    -body	{
	hmac haval256 password {}
    }
    -result A5149401329072996A79A16109346BF546AA5DB74705D5D54A6B7FDF8349AF62
}

::tcltest::test hmac-13.2 "haval256" {
    -body	{
	hmac haval256 password a
    }
    -result 57F4C626C06B3E30C1D3624D3E48576815C53AD6DD540FAB4924C4D6ABCA4A45
}

::tcltest::test hmac-13.3 "haval256" {
    -body	{
	hmac haval256 password abc
    }
    -result C811CC8A02B2A31F4E57F3847EEF7854AE7833FFD9A2A058C2B657D2F560E749
}

::tcltest::test hmac-13.4 "haval256" {
    -body	{
	hmac haval256 password "check the diagonal"
    }
    -result D07910EB2A8BD96434716964D9FA6810DB1BC27788DA474D7F1657600F78897B
}

::tcltest::test hmac-13.5 "haval256" {
    -body	{
	hmac haval256 password "and there was trouble taking place..."
    }
    -result 8E6F5A02E5DEF9579F57F34B212F1012F4D6D7C5F831862032B3C939D43C3C7A
}


#PAGE

::tcltest::test keygen-1.1 "MCRYPT key generation" {
    -body	{
	set data [$mhash keygen mcrypt ciao 100 -hash1 md5]
	binary scan $data H[expr {2*[string length $data]}] data
	set data
    }
    -result 6e6bc4e49dd477ebc98ef4046c067b5f50c2b12ad4f62751baf073656b5d7fbe961f6fdc59fab0063d7ec343a4117b3f0897fbfef1fb08c3773e91255ffeec7dcddbb77cb443331817f946b0def5038e7b5294716eda9fb155e263d67ec957a118de5312
    -cleanup { unset -nocomplain -- data }
}

::tcltest::test keygen-1.2 "MCRYPT key generation, hash options" {
    -body	{
	set data [$mhash keygen mcrypt passphrase 20 -hash1 haval256]
	binary scan $data H[expr {2*[string length $data]}] data
	set data
    }
    -result b24505a79653085bfc02fb7783cb1120445fc751
    -cleanup	{ unset -nocomplain -- data }
}

::tcltest::test keygen-1.3 "MCRYPT key generation, hash options" {
    -body	{
	set data [$mhash keygen mcrypt passphrase 20 -salt 1234 -hash1 sha1]
		binary scan $data H[expr {2*[string length $data]}] data
	set data
    }
    -result 9f1d36ead79194ce6134bf03f0da3b4a9595ab87
    -cleanup	{ unset -nocomplain -- data }
}

::tcltest::test keygen-1.4 "s2k_isalted key generation, hash options" {
    -body	{
	set data [$mhash keygen s2k_isalted passphrase 20 \
		-salt 12345678 -count 10]
	binary scan $data H[expr {2*[string length $data]}] data
	set data
    }
    -result 444d79ef8c4adc2df807ee8caa5f6603594f9d48
    -cleanup	{ unset -nocomplain -- data }
}

::tcltest::test keygen-1.5 "asis key generation, hash options" {
    -body	{
	$mhash keygen asis passphrase [string length passphrase]
    }
    -result passphrase
    -cleanup	{ unset -nocomplain -- data }
}

::tcltest::test keygen-1.6 "hex key generation, hash options" {
    -body	{
	set data [$mhash keygen hex 0123 2]
	binary scan $data H[expr {2*[string length $data]}] data
	set data
    }
    -result 0123
    -cleanup	{ unset -nocomplain -- data }
}

::tcltest::test keygen-1.7 "pkdes key generation, hash options" {
    -body	{
	set data [$mhash keygen pkdes ciao 4]
	binary scan $data H[expr {2*[string length $data]}] data
	set data
    }
    -result e3e961ef
    -cleanup	{ unset -nocomplain -- data }
}

#PAGE

::tcltest::test fcopy-1.1 "hash file" {
    -setup	{
	set data [string repeat a 16000]
	deleteFile
    }
    -body	{
	string equal [hashFile md5 $data] [fcopyHashFile md5 $data]
    }
    -result	1
    -cleanup	{ deleteFile }
}

::tcltest::test fcopy-1.2 "hash file" {
    -setup	{
	set algo sha1
	set data [string repeat a 100000]
	deleteFile
    }
    -body	{
	string equal [hashFile $algo $data] [fcopyHashFile $algo $data]
    }
    -result	1
    -cleanup	{ deleteFile }
}

#PAGE

::tcltest::test keygen-2.1 "pkdes" {
    -body	{
	keygen pkdes ciao
    }
    -result E3E961EF808080808080
}

::tcltest::test keygen-2.2 "pkdes" {
    -body	{
	keygen pkdes 0123456789
    }
    -result B03132B334B5B63738B9
}

#PAGE

::tcltest::test keygen-3.1 "s2k_simple" {
    -body	{
	keygen s2k_simple ciao
    }
    -result 6E6BC4E49DD477EBC98E
}

::tcltest::test keygen-3.2 "s2k_simple" {
    -body	{
	keygen s2k_simple 0123456789
    }
    -result 781E5E245D69B566979B
}

#PAGE

::tcltest::test keygen-4.1 "s2k_salted" {
    -body	{
	keygen s2k_salted ciao -salt 12345678 -hash1 md5 -hash2 md4
    }
    -result BCCA07377C17B6D776E4
}

::tcltest::test keygen-4.2 "s2k_salted" {
    -body	{
	keygen s2k_salted 0123456789 -salt 12345678 -hash1 md5 -hash2 md4
    }
    -result C436B9F69716C4F3954B
}

#PAGE

::tcltest::test keygen-5.1 "s2k_isalted" {
    -body	{
	keygen s2k_isalted ciao -salt 12345678 -hash1 md5 -hash2 md4
    }
    -result D3B364287B0E91BB1BB2
}

::tcltest::test keygen-5.2 "s2k_isalted" {
    -body	{
	keygen s2k_isalted 0123456789 -salt 12345678 -hash1 md5 -hash2 md4
    }
    -result 071D6DDA061B732C0774
}

#PAGE

::tcltest::test error-1.1 "wrong num arguments" {
    -body	{
	catch {$mhash} res
	list $res $errorCode
    }
    -result "{wrong # args: should be \"$mhash subcommand ?arg ...?\"} LOGIC"
}

::tcltest::test error-1.2 "wrong subcommand" {
    -body	{
	catch {$mhash gasp} res
	list $res $errorCode
    }
    -result "{bad subcommand \"gasp\": must be *} LOGIC"
    -match glob
}

::tcltest::test error-1.3 "wrong num args"  \
    -body	{
	catch {$mhash hash} res
        list $res $::errorCode
    } \
    -result [list "wrong # args: should be \"$mhash hash algorithm\"" "LOGIC"]


::tcltest::test error-1.4 "wrong num args" \
    -body	{
	catch {$mhash hash gasp gulp} res
	list $res $::errorCode
    } \
    -result [list "wrong # args: should be \"$mhash hash algorithm\"" "LOGIC"]


::tcltest::test error-1.5 "unexistent algorithm" {
    -body	{
	catch {$mhash hash gulp} res
	list $res $::errorCode
    }
    -result "{bad algorithm \"gulp\": must be *} LOGIC"
    -match glob
}

::tcltest::test error-1.6 "wrong num args" \
    -body	{
	catch {$mhash hmac} res
	list $res $::errorCode
    } \
    -result [list "wrong # args: should be \"$mhash hmac algorithm password\"" "LOGIC"]


::tcltest::test error-1.7 "wrong num args" \
    -body	{
	catch {$mhash hmac gasp gulp glub} res
	list $res $::errorCode
    } \
    -result [list "wrong # args: should be \"$mhash hmac algorithm password\"" "LOGIC"]


::tcltest::test error-1.8 "unexistent algorithm" {
    -body	{
	catch {$mhash hmac gulp passphrase} res
	list $res $::errorCode
    }
    -result "{bad algorithm \"gulp\": must be *} LOGIC"
    -match glob
}

::tcltest::test error-1.9 "null password" {
    -body	{
	catch {$mhash hmac tiger {}} res
	list $res $::errorCode
    }
    -result "{expected non-empty block} LOGIC"
}

::tcltest::test error-1.17 "wrong num args" \
    -body	{
	catch {$mhash keygen} res
	list $res $::errorCode
    } \
    -result [list "wrong # args: should be \"$mhash keygen algorithm password length ?options?\"" "LOGIC"]


::tcltest::test error-1.18 "unexistent algorithm" {
    -body	{
	catch {$mhash keygen gasp passphrase 10} res
	list $res $::errorCode
    }
    -result {{bad algorithm "gasp": *} LOGIC}
    -match glob
}

::tcltest::test error-1.19 "requested null key" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 0} res
	list $res $::errorCode
    }
    -result {{requested null key} LOGIC}
}

::tcltest::test error-1.20 "bad option" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -glab} res
	list $res $::errorCode
    }
    -result {{bad option "-glab": *} LOGIC}
    -match glob
}

::tcltest::test error-1.21 "bad option value" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -hash1 gasp} res
	list $res $::errorCode
    }
    -result {{bad algorithm "gasp": *} LOGIC}
    -match glob
}

::tcltest::test error-1.22 "bad option value" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -hash2 gasp} res
	list $res $::errorCode
    }
    -result {{bad algorithm "gasp": *} LOGIC}
    -match glob
}

::tcltest::test error-1.23 "bad option value" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -count gasp} res
	list $res $::errorCode
    }
    -result {{expected unsigned but got "gasp"} LOGIC}
}

::tcltest::test error-1.24 "option requires argument" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -salt} res
	list $res $::errorCode
    }
    -result {{option "-salt" requires argument} LOGIC}
    -match glob
}

::tcltest::test error-1.25 "option requires argument" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -count} res
	list $res $::errorCode
    }
    -result {{option "-count" requires argument} LOGIC}
    -match glob
}

::tcltest::test error-1.26 "option requires argument" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -hash1} res
	list $res $::errorCode
    }
    -result {{option "-hash1" requires argument} LOGIC}
    -match glob
}

::tcltest::test error-1.27 "option requires argument" {
    -body	{
	catch {$mhash keygen s2k_isalted passphrase 10 -hash2} res
	list $res $::errorCode
    }
    -result {{option "-hash2" requires argument} LOGIC}
    -match glob
}

::tcltest::test error-1.28 "hex key generation, wrong password length" {
    -body	{
	catch { $mhash keygen hex 012 2 } res
	list $res $::errorCode
    }
    -result "{when using the hex algorithm the requested key length must be half of the password length} LOGIC"
}

::tcltest::test error-1.29 "hex key generation, wrong key length" {
    -body	{
	catch { $mhash keygen hex 0123 3 } res
	list $res $::errorCode
    }
    -result "{when using the hex algorithm the requested key length must be half of the password length} LOGIC"
    -cleanup	{ unset -nocomplain -- data }
}

#PAGE

::tcltest::test info-1.1 "hash algorithm existence" {
    -body	{
	$mhash info hash exists md5
    }
    -result {[01]}
    -match glob
}

::tcltest::test info-1.2 "hash exists, unknown algorithm" {
    -body	{
	catch {$mhash info hash exists gulp} res
	set res
    }
    -result "bad algorithm \"gulp\": *"
    -match glob
}

::tcltest::test info-1.3 "hash exists, wrong num args" {
    -body	{
	catch {$mhash info hash exists} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash exists algorithm\""
}

::tcltest::test info-1.4 "hash exists, wrong num args" {
    -body	{
	catch {$mhash info hash exists md5 gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash exists algorithm\""
}

#PAGE

::tcltest::test info-2.1 "is HMAC algorithm?" {
    -body	{
	$mhash info hash ishmacalgo gost
    }
    -result {[01]}
    -match glob
}

::tcltest::test info-2.2 "is HMAC algo, unknown algorithm" {
    -body	{
	catch {$mhash info hash ishmacalgo gulp} res
	set res
    }
    -result "bad algorithm \"gulp\": *"
    -match glob
}

::tcltest::test info-2.3 "hash exists, wrong num args" {
    -body	{
	catch {$mhash info hash ishmacalgo} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash ishmacalgo algorithm\""
}

::tcltest::test info-2.4 "hash exists, wrong num args" {
    -body	{
	catch {$mhash info hash ishmacalgo md5 gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash ishmacalgo algorithm\""
}

#PAGE

::tcltest::test info-3.1 "list hash algorithms" {
    -body	{
	$mhash info hash list
    }
    -result *
    -match glob
}

::tcltest::test info-3.2 "list hash algorithms, wrong num args" {
    -body	{
	catch {$mhash info hash list md5} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash list\""
}

#PAGE

::tcltest::test info-4.1 "hash block size" {
    -body	{
	$mhash info hash blocksize md5
    }
    -result {[0-9]+}
    -match regexp
}

::tcltest::test info-4.2 "hash block size, unknown algorithm" {
    -body	{
	catch {$mhash info hash blocksize gulp} res
	set res
    }
    -result "bad algorithm \"gulp\": *"
    -match glob
}

::tcltest::test info-4.3 "hash block size, wrong num args" {
    -body	{
	catch {$mhash info hash blocksize} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash blocksize algorithm\""
}

::tcltest::test info-4.4 "hash block size, wrong num args" {
    -body	{
	catch {$mhash info hash blocksize md5 gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info hash blocksize algorithm\""
}

#PAGE

::tcltest::test info-5.1 "keygen exists" {
    -body	{
	$mhash info keygen exists mcrypt
    }
    -result 1
}

::tcltest::test info-5.2 "keygen exists, bad algorithm" {
    -body	{
	catch {$mhash info keygen exists gasp} res
	set res
    }
    -result "bad algorithm \"gasp\": *"
    -match glob
}

::tcltest::test info-5.3 "keygen exists, wrong num args" {
    -body	{
	catch {$mhash info keygen exists} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen exists algorithm\""
}

::tcltest::test info-5.4 "keygen exists, wrong num args" {
    -body	{
	catch {$mhash info keygen exists mcrypt gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen exists algorithm\""
}

#PAGE

::tcltest::test info-6.1 "list hash" {
    -body	{
	$mhash info keygen list
    }
    -result *
    -match glob
}

::tcltest::test info-6.2 "list hash, wrong num args" {
    -body	{
	catch {$mhash info keygen list mcrypt} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen list\""
}

#PAGE

::tcltest::test info-7.1 "keygen uses count" {
    -body	{
	$mhash info keygen usescount mcrypt
    }
    -result 0
}

::tcltest::test info-7.2 "keygen uses count" {
    -body	{
	$mhash info keygen usescount s2k_isalted
    }
    -result 1
}

::tcltest::test info-7.3 "keygen uses count, bad algorithm" {
    -body	{
	catch {$mhash info keygen usescount gasp} res
	set res
    }
    -result "bad algorithm \"gasp\": *"
    -match glob
}

::tcltest::test info-7.4 "keygen uses count, wrong num args" {
    -body	{
	catch {$mhash info keygen usescount} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen usescount algorithm\""
}

::tcltest::test info-7.5 "keygen uses count, wrong num args" {
    -body	{
	catch {$mhash info keygen usescount mcrypt gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen usescount algorithm\""
}

#PAGE

::tcltest::test info-8.1 "keygen uses hash" {
    -body	{
	$mhash info keygen useshash mcrypt
    }
    -result 1
}

::tcltest::test info-8.2 "keygen uses hash" {
    -body	{
	$mhash info keygen useshash pkdes
    }
    -result 0
}

::tcltest::test info-8.3 "keygen uses hash" {
    -body	{
	$mhash info keygen useshash hex
    }
    -result 0
}

::tcltest::test info-8.4 "keygen uses hash" {
    -body	{
	$mhash info keygen useshash s2k_isalted
    }
    -result 1
}

::tcltest::test info-8.5 "keygen uses hash" {
    -body	{
	$mhash info keygen useshash s2k_salted
    }
    -result 1
}

::tcltest::test info-8.5 "keygen uses hash" {
    -body	{
	$mhash info keygen useshash s2k_simple
    }
    -result 1
}

::tcltest::test info-8.6 "keygen uses hash, bad algorithm" {
    -body	{
	catch {$mhash info keygen useshash gasp} res
	set res
    }
    -result "bad algorithm \"gasp\": *"
    -match glob
}

::tcltest::test info-8.7 "keygen uses hash, wrong num args" {
    -body	{
	catch {$mhash info keygen useshash} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen useshash algorithm\""
}

::tcltest::test info-8.8 "keygen uses hash, wrong num args" {
    -body	{
	catch {$mhash info keygen useshash mcrypt gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen useshash algorithm\""
}

#PAGE

::tcltest::test info-9.1 "keygen uses salt" {
    -body	{
	$mhash info keygen usessalt hex
    }
    -result 0
}

::tcltest::test info-9.2 "keygen uses salt" {
    -body	{
	$mhash info keygen usessalt s2k_isalted
    }
    -result 1
}

::tcltest::test info-9.3 "keygen uses salt, bad algorithm" {
    -body	{
	catch {$mhash info keygen usessalt gasp} res
	set res
    }
    -result "bad algorithm \"gasp\": *"
    -match glob
}

::tcltest::test info-9.4 "keygen uses salt, wrong num args" {
    -body	{
	catch {$mhash info keygen usessalt} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen usessalt algorithm\""
}

::tcltest::test info-9.5 "keygen uses salt, wrong num args" {
    -body	{
	catch {$mhash info keygen usessalt mcrypt gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen usessalt algorithm\""
}

#PAGE

::tcltest::test info-10.1 "keygen salt size" {
    -body	{
	$mhash info keygen saltsize s2k_isalted
    }
    -result {[0-9]+}
    -match regexp
}

::tcltest::test info-10.2 "keygen salt size, bad algorithm" {
    -body	{
	catch {$mhash info keygen saltsize gasp} res
	set res
    }
    -result "bad algorithm \"gasp\": *"
    -match glob
}

::tcltest::test info-10.3 "keygen salt size, wrong num args" {
    -body	{
	catch {$mhash info keygen saltsize} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen saltsize algorithm\""
}

::tcltest::test info-10.4 "keygen salt size, wrong num args" {
    -body	{
	catch {$mhash info keygen saltsize mcrypt gasp} res
	set res
    }
    -result "wrong # args: should be \"$mhash info keygen saltsize algorithm\""
}

#PAGE

::tcltest::test info-11.1 "version" {
    -body	{
	$mhash info version
    }
    -result {[0-9]+}
    -match regexp
}

::tcltest::test info-11.2 "version, wrong num args" {
    -body	{
	catch {$mhash info version 1} res
	set res
    }
    -result "wrong # args: should be \"$mhash info version\""
}

#PAGE

::tcltest::test info-12.1 "info wrong num args" {
    -body	{
	catch {$mhash info} res
	set res
    }
    -result "wrong # args: should be \"$mhash info subcommand ?arg ...?\""
}

#PAGE

::tcltest::test info-13.1 "keygen key size" {
    -body	{
	$mhash info keygen maxkeysize s2k_isalted
    }
    -result {[0-9]+}
    -match regexp
}


#PAGE
## ---------------------------------------------------------
## End of "::test" namespace.
## ---------------------------------------------------------

::tcltest::cleanupTests
}
namespace delete ::test


### end of file
# Local Variables:
# mode: tcl
# page-delimiter: "^#PAGE"
# End:
